
package Controls;

import java.awt.*;
import java.awt.event.*;

/** Ein Control-Objekt zur Auswahl eines Items aus einer festen Menge. */

public class Auswahl extends CtrlBox implements ItemListener {

	private Choice choice;

	private CB  cb;
	private int id;

	/**
	 * Erzeugt ein Auswahl-Objekt mit Name, Item-Liste, Callback, Rand und Farben.
	 *
	 * @param name	Name der Box.
	 * @param items	Liste der einzutragenden Items.
	 * @param cb	Master-Objekt fuer Callback (null -> kein EventListener).
	 * @param id	Master-ID fuer Callback.
	 * @param rand	Rand oder kein Rand.
	 * @param fg	Vordergrund-Farbe der Box.
	 * @param bg	Hintergrund-Farbe der Box.
	 * @param sfg	Vordergrund-Farbe der internen Komponenten.
	 * @param sbg	Hintergrund-Farbe der internen Komponenten.
	 */

	public Auswahl(String name, String[] items, CB cb, int id, boolean rand, Color fg, Color bg, Color sfg, Color sbg) {

		super(name, rand ? RAND : NONE, fg, bg);

		this.cb = cb;
		this.id = id;

		choice = new Choice();

		choice.setForeground(sfg == null ? FG : sfg);
		choice.setBackground(sbg == null ? BG : sbg);

		for (int i=0; i < items.length; i++) {

			choice.addItem(items[i]);
		}

		if (cb != null) choice.addItemListener(this);

		addBag(new Label(name + ":", Label.CENTER),
			0, 0, 1, 1, E, PAD, PAD, PAD, 0);
		addBag(choice,
			1, 0, 1, 1, W, PAD, PAD, 0, 2*PAD);
	}

	/** Erzeugt ein Auswahl-Objekt mit Default-Farben. */

	public Auswahl(String name, String[] items, CB cb, int id, boolean rand) {

		this(name, items, cb, id, rand, null, null, null, null);
	}

	/** Erzeugt ein Auswahl-Objekt mit Rand. */

	public Auswahl(String name, String[] items, CB cb, int id, Color fg, Color bg, Color sfg, Color sbg) {

		this(name, items, cb, id, true, fg, bg, sfg, sbg);
	}

	/** Erzeugt ein Auswahl-Objekt mit Rand und Default-Farben. */

	public Auswahl(String name, String[] items, CB cb, int id) {

		this(name, items, cb, id, true, null, null, null, null);
	}

	/** Erzeugt ein Auswahl-Objekt mit Rand und Default-Farben ohne Callback. */

	public Auswahl(String name, String[] items) {

		this(name, items, null, 0, true, null, null, null, null);
	}

	/**
	 * EventListener-Methode.
	 *
 	 * Der zugehoerige Callback wird aufgerufen.
	 * Argument: Ausgewaehltes Item.
	 */

	public void itemStateChanged(ItemEvent e) {

		cb.callback(id, this, choice.getSelectedItem());
	}

	/**
	 * Setzt das aktive Item.
	 *
	 * @param item Neues aktives Item.
	 */

	public void setActiveItem(String item) {

		choice.select(item);
	}

	/**
	 * Liefert das aktive Item.
	 *
	 * @return Gibt das aktive Item als String zurueck.
	 */

	public String getActiveItem() {

		return(choice.getSelectedItem());
	}

	/**
	 * Aendert die Farben des Auswahl-Objekts.
	 *
	 * @param fg   Vordergrund-Farbe der Box.
	 * @param bg   Hintergrund-Farbe der Box.
	 * @param sfg  Vordergrund-Farbe der internen Komponenten.
	 * @param sbg  Hintergrund-Farbe der internen Komponenten.
	 */

	public void chgColors(Color fg, Color bg, Color sfg, Color sbg) {

		chgBoxColors(fg, bg);

		choice.setForeground(sfg == null ? FG : sfg);
		choice.setBackground(sbg == null ? BG : sbg);
	}
}
