
package Controls;

import java.awt.*;

/** Abstraktes Standard-Control-Element, von dem alle Controls abstammen. */

public abstract class CtrlBox extends Panel implements Ctrl {

	private String name;
	private int    rand;
	private int    xpad;

	private Color  box_fg;
	private Color  box_bg;

	/**
	 * Erzeugt ein CtrlBox-Objekt mit Name, Rand-Typ und Farben.
	 *
	 * @param name	Name der Box.
	 * @param rand	Rand-Typ der Box.
	 * @param fg	Vordergrund-Farbe der Box.
	 * @param bg	Hintergrund-Farbe der Box.
	 */

	public CtrlBox(String name, int rand, Color fg, Color bg) {

		super();

		this.name = name;
		this.rand = rand;

		this.xpad = rand == NAME ? 2 * PAD : 0;

		this.box_fg = fg == null ? FG : fg;
		this.box_bg = bg == null ? BG : bg;

		setForeground(box_fg);
		setBackground(box_bg);

		setLayout(new GridBagLayout());
	}

	/** Paint-Methode, die eine Box mit einem passenden Rand zeichnet. */

	public void paint(Graphics g) {

		super.paint(g);

		Rectangle r = getBounds();
		Color bak_c = g.getColor();

		g.setColor(box_fg);

		switch (rand) {

			case NONE: // do nothing
				break;

			case RAND:
				g.draw3DRect(0, 0, r.width - 1, r.height - 1, true);
				break;

			case NAME:
				g.draw3DRect(0, 0, r.width - 1, r.height - 1, true);
				g.drawString(name, PAD, 2 * PAD);

				break;

			default:
				System.err.println("CtrlBox-Error: wrong rand-label: " + rand);
		}

		g.setColor(bak_c);
	}

	/** Liefert den Namen der Box */

	public String getName() {

		return (name);
	}

	/**
	 * Aendert die Farben der Box.
	 *
	 * @param fg   Vordergrund-Farbe der Box.
	 * @param bg   Hintergrund-Farbe der Box.
	 */

	protected void chgBoxColors(Color fg, Color bg) {

		this.box_fg = fg == null ? FG : fg;
		this.box_bg = bg == null ? BG : bg;

		setForeground(box_fg);
		setBackground(box_bg);
	}

	/**
	 * Fuegt eine Komponente in die CtrlBox ein.
	 *
	 * @param comp	Einzufuegende Komponente.
	 * @param x	X-Position der Komponente.
	 * @param y	Y-Position der Komponente.
	 * @param w	Horizontale Ausdehnung der Komponente.
	 * @param h	Vertikale Ausdehnung der Komponente.
	 * @param anchor Ausrichtung der Komponente.
	 * @param fill	Ausdehnung der Komponente (allgemeines Verhalten).
	 * @param w_x	Ausdehnung der Komponente (bei Vergroesserung in X-Dimension).
	 * @param w_y	Ausdehnung der Komponente (bei Vergroesserung in Y-Dimension).
	 * @param top	Rand oberhalb der Komponente.
	 * @param bot	Rand unterhalb der Komponente.
	 * @param l	Rand links von der Komponente.
	 * @param r	Rand rechts von der Komponente.
	 */

	protected void addBag(Component comp,
				int x, int y, int w, int h,
				int anchor, int fill,
				double w_x, double w_y,
				int top, int bot, int l, int r) {

		GridBagConstraints c = new GridBagConstraints();

		c.gridx = x;
		c.gridy = y;
		c.gridwidth  = w;
		c.gridheight = h;
		c.anchor = anchor;
		c.fill = fill;		
		c.weightx = w_x;
		c.weighty = w_y;

		c.insets = new Insets(top + xpad, l, bot, r);

		((GridBagLayout) this.getLayout()).setConstraints(comp, c);

		this.add(comp);
	}

	/** Fuegt eine Komponente ohne Ausdehnungs-Aenderung in die CtrlBox ein. */

	protected void addBag(Component comp, int x, int y, int w, int h, int anchor,
				int top, int bot, int l, int r) {

		addBag(comp, x, y, w, h, anchor,
			GridBagConstraints.NONE, 0.0, 0.0, 
			top, bot, l, r);
	}

	/** Fuegt eine Komponente ohne Ausdehnungs-Aenderung mit Standard-Rand in die CtrlBox ein. */

	protected void addBag(Component comp, int x, int y, int w, int h, int anchor) {

		addBag(comp, x, y, w, h, anchor, PAD, PAD, PAD, PAD);
	}

	/** Fuegt eine Komponente ohne Ausdehnungs-Aenderung mit Standard-Rand zentriert in die CtrlBox ein. */

	protected void addBag(Component comp, int x, int y, int w, int h) {

		addBag(comp, x, y, w, h, GridBagConstraints.CENTER);
	}
}
