
package Controls;

import java.awt.*;
import java.awt.event.*;

/** Ein Control-Objekt zur Darstellung einer CtrlBox in einem separaten Fenster. */

public class CtrlFrame extends Frame implements WindowListener, Ctrl {

	private CB  cb;
	private int id;

	/**
	 * Erzeugt ein CtrlFrame-Objekt mit Name, Box und Callback.
	 *
	 * @param name	Name des Fensters.
	 * @param box	Einzufuegende CtrlBox.
	 * @param cb	Master-Objekt fuer Callback.
	 * @param id	Master-ID fuer Callback.
	 */

	public CtrlFrame(String title, CtrlBox box, CB cb, int id) {

		super(title);

		this.cb = cb;
		this.id = id;

		add(box, "Center");
		addWindowListener(this);

        	pack();
        	show();
	}

	/**
	 * EventListener-Methode fuer Window-Closing.
	 *
 	 * Der zugehoerige Callback wird aufgerufen.
	 * Falls das Master-Object null ist, wird System.exit(id) aufgerufen.
	 */

	public void windowClosing(WindowEvent e) {

		if (cb == null) {

			System.exit(id);

		} else {

			cb.callback(id, this, null);
		}
	}

	/** Dummy EventListener-Methode. */
	public void windowOpened     (WindowEvent e) {}
	/** Dummy EventListener-Methode. */
	public void windowClosed     (WindowEvent e) {}
	/** Dummy EventListener-Methode. */
	public void windowActivated  (WindowEvent e) {}
	/** Dummy EventListener-Methode. */
	public void windowDeactivated(WindowEvent e) {}
	/** Dummy EventListener-Methode. */
	public void windowIconified  (WindowEvent e) {}
	/** Dummy EventListener-Methode. */
	public void windowDeiconified(WindowEvent e) {}
}
