
package Controls;

import java.awt.*;

/** Ctrl-Element zum Zusammenbauen von CtrlBoxen. */

public class CtrlPanel extends CtrlBox {

	private int orient;
	private int pos;

	/**
	 * Erzeugt ein CtrlPanel-Objekt mit Name, Orientierung, Rand und Farben.
	 *
	 * @param name	Name der Box.
	 * @param orient Orientierung des CtrlPanel (Komponenten werden neben- oder uebereinander eingefuegt).
	 * @param rand	Rand-Typ der Box.
	 * @param fg	Vordergrund-Farbe der Box.
	 * @param bg	Hintergrund-Farbe der Box.
	 */

	public CtrlPanel(String name, int orient, int rand, Color fg, Color bg) {

		super(name, rand, fg, bg);

		this.orient = orient;

		pos = 0;
	}

	/** Erzeugt ein CtrlPanel-Objekt mit Default-Farben. */

	public CtrlPanel(String name, int orient, int rand) {


		this(name, orient, rand, null, null);
	}

	/** Erzeugt ein horizontales CtrlPanel-Objekt ohne Rand. */

	public CtrlPanel(String name, Color fg, Color bg) {

		this(name, HORZ, NONE, fg, bg);
	}

	/** Erzeugt ein horizontales CtrlPanel-Objekt ohne Rand mit Default-Farben. */

	public CtrlPanel(String name) {

		this(name, HORZ, NONE, null, null);
	}

	/**
	 * Fuegt eine Komponente in das CtrlPanel ein.
	 *
	 * @param item	Einzufuegende Komponente.
	 * @param anchor Ausrichtung der Komponente.
	 * @param last Flag fuer letzte Komponente.
	 */

	public void addItem(Component item, int anchor, boolean last) {
	
		if (orient == HORZ) { 

			addBag(item, pos, 0, 1, 1, anchor, PAD, PAD, PAD, last ? PAD : 0);

		} else {   // VERT

			addBag(item, 0, pos, 1, 1, anchor, PAD, last ? PAD : 0, PAD, PAD);
		}

		pos++;
	}

	/**
	 * Fuegt eine Komponente in das CtrlPanel ein.
	 *
	 * @param item	Einzufuegende Komponente.
	 * @param anchor Ausrichtung der Komponente.
	 */

	public void addItem(Component item, int anchor) {

		addItem(item, anchor, false);	
	}

	/**
	 * Fuegt eine Komponente mit Default-Ausrichtung in das CtrlPanel ein.
	 *
	 * @param item Einzufuegende Komponente.
	 * @param last Flag fuer letzte Komponente.
	 */

	public void addItem(Component item, boolean last) {

		if (orient == HORZ) { 

			addItem(item, W, last);

		} else {   // VERT

			addItem(item, N, last);
		}
	}

	/**
	 * Fuegt eine Komponente mit Default-Ausrichtung in das CtrlPanel ein.
	 *
	 * @param item	Einzufuegende Komponente.
	 */

	public void addItem(Component item) {

		addItem(item, false);
	}

	/**
	 * Fuegt die letzte Komponente in das CtrlPanel ein.
	 *
	 * @param item	Einzufuegende Komponente.
	 * @param anchor Ausrichtung der Komponente.
	 */

	public void addLast(Component item, int anchor) {

		addItem(item, anchor, true);
	}

	/**
	 * Fuegt die letzte Komponente mit Default-Ausrichtung in das CtrlPanel ein.
	 *
	 * @param item	Einzufuegende Komponente.
	 */

	public void addLast(Component item) {

		if (orient == HORZ) { 

			addLast(item, W);

		} else {   // VERT

			addLast(item, N);
		}
	}

	/**
	 * Loescht die angegebene Komponente.
	 *
	 * @param item	    Komponente.
	 * @param valGlobal validate complete Application?.
	 */

	public void delItem(Component item, boolean valGlobal) {

		remove(item);

		if (valGlobal)	{ Tools.validateApp(this); }
		else		{ validate(); }
	}

	/**
	 * Loescht die angegebene Komponente.
	 *
	 * @param item Komponente.
	 */

	public void delItem(Component item) {

		delItem(item, true);
	}

	/**
	 * Loescht die angegebene Komponente.
	 *
	 * @param name	    Name der Komponente.
	 * @param valGlobal validate complete Application?.
	 */

	public void delItem(String name, boolean valGlobal) {

		Component item = getItem(name);

		if (item == null) {

			System.err.println("CtrlPanel.delItem():: Component " + name + " not found");

		} else {

			remove(item);

			if (valGlobal)	{ Tools.validateApp(this); }
			else		{ validate(); }
		}
	}

	/**
	 * Loescht die angegebene Komponente.
	 *
	 * @param name Name der Komponente.
	 */

	public void delItem(String name) {

		delItem(name, true);
	}

	/**
	 * Macht die angegebene Komponente unsichtbar.
	 *
	 * @param item	    Komponente.
	 * @param valGlobal validate complete Application?.
	 */

	public void hideItem(Component item, boolean valGlobal) {

		item.setVisible(false);

		if (valGlobal)	{ Tools.validateApp(this); }
		else		{ validate(); }
	}

	/**
	 * Macht die angegebene Komponente unsichtbar.
	 *
	 * @param name	    Name der Komponente.
	 * @param valGlobal validate complete Application?.
	 */

	public void hideItem(String name, boolean valGlobal) {

		Component item = getItem(name);

		if (item == null) {

			System.err.println("CtrlPanel.hideItem():: Component " + name + " not found");

		} else {

			item.setVisible(false);

			if (valGlobal)	{ Tools.validateApp(this); }
			else		{ validate(); }
		}
	}

	/**
	 * Macht die angegebene Komponente unsichtbar.
	 *
	 * @param item Komponente.
	 */

	public void hideItem(Component item) {

		hideItem(item, true);
	}

	/**
	 * Macht die angegebene Komponente unsichtbar.
	 *
	 * @param name Name der Komponente.
	 */

	public void hideItem(String name) {

		hideItem(name, true);
	}

	/**
	 * Macht die angegebene Komponente sichtbar.
	 *
	 * @param item	    Komponente.
	 * @param valGlobal validate complete Application?.
	 */

	public void showItem(Component item, boolean valGlobal) {

		item.setVisible(true);

		if (valGlobal)	{ Tools.validateApp(this); }
		else		{ validate(); }
	}

	/**
	 * Macht die angegebene Komponente sichtbar.
	 *
	 * @param item Komponente.
	 */

	public void showItem(Component item) {

		showItem(item, true);
	}

	/**
	 * Macht die angegebene Komponente sichtbar.
	 *
	 * @param name	    Name der Komponente.
	 * @param valGlobal validate complete Application?.
	 */

	public void showItem(String name, boolean valGlobal) {

		Component item = getItem(name);

		if (item == null) {

			System.err.println("CtrlPanel.showItem():: Component " + name + " not found");

		} else {

			item.setVisible(true);

			if (valGlobal)	{ Tools.validateApp(this); }
			else		{ validate(); }
		}
	}

	/**
	 * Macht die angegebene Komponente sichtbar.
	 *
	 * @param name Name der Komponente.
	 */

	public void showItem(String name) {

		showItem(name, true);
	}
	
	/**
	 * Liefert die Komponente mit dem angegebenen Namen.
	 *
	 * @param name Name der Komponente.
	 */

	public Component getItem(String name) {

		Component[] all = getComponents();

		for (int i=0; i<all.length; i++) {

			if (all[i].getName().equals(name)) {

				return (all[i]);
			}
		}

		return (null);
	}

	/**
	 * Aendert die Farben des CtrlPanel-Objekts.
	 *
	 * @param fg   Vordergrund-Farbe der Box.
	 * @param bg   Hintergrund-Farbe der Box.
	 */

	public void chgColors(Color fg, Color bg) {

		chgBoxColors(fg, bg);
	}
}
