
package Controls;

import java.awt.*;
import java.awt.event.*;

/** Ein Control-Objekt zur Eingabe eines einzeiligen Textes. */

public class Eingabe extends CtrlBox implements ActionListener {

	private TextField ein;

	private CB  cb;
	private int id;

	/**
	 * Erzeugt ein Eingabe-Objekt mit Name, Text, Groesse, Callback, Rand und Farben.
	 *
	 * @param name	Name der Box.
	 * @param text	Liste der einzutragenden Items.
	 * @param cols	Anzahl Spalten.
	 * @param cb	Master-Objekt fuer Callback (null -> kein EventListener).
	 * @param id	Master-ID fuer Callback.
	 * @param rand	Rand oder kein Rand.
	 * @param fg	Vordergrund-Farbe der Box.
	 * @param bg	Hintergrund-Farbe der Box.
	 * @param sfg	Vordergrund-Farbe der internen Komponenten.
	 * @param sbg	Hintergrund-Farbe der internen Komponenten.
	 */

	public Eingabe(String name, String text, int cols, CB cb, int id, boolean rand, Color fg, Color bg, Color sfg, Color sbg) {

		super(name, rand ? RAND : NONE, fg, bg);

		this.cb = cb;
		this.id = id;

		ein = new TextField(text, cols);

		ein.setForeground(sfg == null ? FG : sfg);
		ein.setBackground(sbg == null ? BG : sbg);

		if (cb != null) ein.addActionListener(this);

		addBag(new Label(name + ":", Label.CENTER),
			0, 0, 1, 1, E, PAD, PAD, PAD, 0);
		addBag(ein,
			1, 0, 1, 1, W, PAD, PAD, 0, PAD);
	}

	/** Erzeugt ein Eingabe-Objekt mit Default-Farben. */

	public Eingabe(String name, String text, int cols, CB cb, int id, boolean rand) {

		this(name, text, cols, cb, id, rand, null, null, null, null);
	}

	/** Erzeugt ein Eingabe-Objekt mit Rand. */

	public Eingabe(String name, String text, int cols, CB cb, int id, Color fg, Color bg, Color sfg, Color sbg) {

		this(name, text, cols, cb, id, true, fg, bg, sfg, sbg);
	}

	/** Erzeugt ein Eingabe-Objekt mit Rand und Default-Farben. */

	public Eingabe(String name, String text, int cols, CB cb, int id) {

		this(name, text, cols, cb, id, true, null, null, null, null);
	}

	/** Erzeugt ein Eingabe-Objekt mit Rand und Default-Farben ohne Callback. */

	public Eingabe(String name, String text, int cols) {

		this(name, text, cols, null, 0, true, null, null, null, null);
	}

	/**
	 * EventListener-Methode.
	 *
 	 * Der zugehoerige Callback wird aufgerufen.
	 * Argument: Eingegebener Text.
	 */

	public void actionPerformed(ActionEvent e) {

		cb.callback(id, this, ein.getText());
	}

	/**
	 * Setzt den dargestellten Text.
	 *
	 * @param text Neuer dargestellter Text.
	 */

	public void setText(String text) {

		ein.setText(text);
	}

	/**
	 * Liefert den dargestellten Text.
	 *
	 * @return Gibt den dargestellten Texti als String zurueck.
	 */

	public String getText() {

		return(ein.getText());
	}

	/**
	 * Aendert die Farben des Eingabe-Objekts.
	 *
	 * @param fg   Vordergrund-Farbe der Box.
	 * @param bg   Hintergrund-Farbe der Box.
	 * @param sfg  Vordergrund-Farbe der internen Komponenten.
	 * @param sbg  Hintergrund-Farbe der internen Komponenten.
	 */

	public void chgColors(Color fg, Color bg, Color sfg, Color sbg) {

		chgBoxColors(fg, bg);

		ein.setForeground(sfg == null ? FG : sfg);
		ein.setBackground(sbg == null ? BG : sbg);
	}
}
