
package Controls;

import java.awt.*;
import java.awt.event.*;

/** Ein Control-Objekt zur Aktivierung eines Callbacks. */

public class Knopf extends CtrlBox implements ActionListener {

	private Button button;

	private CB  cb;
	private int id;

	/**
	 * Erzeugt ein Knopf-Objekt mit Name, Callback, Rand und Farben.
	 *
	 * @param name	Name der Box.
	 * @param cb	Master-Objekt fuer Callback.
	 * @param id	Master-ID fuer Callback.
	 * @param rand	Rand oder kein Rand.
	 * @param fg	Vordergrund-Farbe der Box.
	 * @param bg	Hintergrund-Farbe der Box.
	 * @param sfg	Vordergrund-Farbe der internen Komponenten.
	 * @param sbg	Hintergrund-Farbe der internen Komponenten.
	 */

	public Knopf(String name, CB cb, int id, boolean rand, Color fg, Color bg, Color sfg, Color sbg) {

		super(name, rand ? RAND : NONE, fg, bg);

		this.cb = cb;
		this.id = id;

		button = new Button(name);

		button.setForeground(sfg == null ? FG : sfg);
		button.setBackground(sbg == null ? BG : sbg);

		if (cb != null) button.addActionListener(this);

		addBag(button, 0, 0, 1, 1);
	}

	/** Erzeugt ein Knopf-Objekt mit Default-Farben. */

	public Knopf(String name, CB cb, int id, boolean rand) {

		this(name, cb, id, rand, null, null, null, null);
	}

	/** Erzeugt ein Knopf-Objekt ohne Rand. */

	public Knopf(String name, CB cb, int id, Color fg, Color bg, Color sfg, Color sbg) {

		this(name, cb, id, false, fg, bg, sfg, sbg);
	}

	/** Erzeugt ein Knopf-Objekt ohne Rand mit Default-Farben. */

	public Knopf(String name, CB cb, int id) {

		this(name, cb, id, false, null, null, null, null);
	}

	/**
	 * EventListener-Methode.
	 *
 	 * Der zugehoerige Callback wird aufgerufen.
	 * Argument: keins.
	 */

	public void actionPerformed(ActionEvent e) {

		cb.callback(id, this, null);
	}

	/**
	 * Aendert die Farben des Knopf-Objekts.
	 *
	 * @param fg   Vordergrund-Farbe der Box.
	 * @param bg   Hintergrund-Farbe der Box.
	 * @param sfg  Vordergrund-Farbe der internen Komponenten.
	 * @param sbg  Hintergrund-Farbe der internen Komponenten.
	 */

	public void chgColors(Color fg, Color bg, Color sfg, Color sbg) {

		chgBoxColors(fg, bg);

		button.setForeground(sfg == null ? FG : sfg);
		button.setBackground(sbg == null ? BG : sbg);
	}
}
