
package Controls;

import java.awt.*;
import java.awt.event.*;

/** Ein Control-Objekt zur Auswahl eines Items aus einer festen Menge. */

public class Radio extends CtrlBox implements ItemListener {

	private CheckboxGroup rbox;
	private Checkbox[] cboxes;

	private CB  cb;
	private int id;

	/**
	 * Erzeugt ein Radio-Objekt mit Name, Item-Liste, Callback, Rand und Farben.
	 *
	 * @param name	Name der Box.
	 * @param items	Liste der einzutragenden Items.
	 * @param cb	Master-Objekt fuer Callback (null -> kein EventListener).
	 * @param id	Master-ID fuer Callback.
	 * @param rand	Rand oder kein Rand.
	 * @param fg	Vordergrund-Farbe der Box.
	 * @param bg	Hintergrund-Farbe der Box.
	 * @param sfg	Vordergrund-Farbe der internen Komponenten.
	 * @param sbg	Hintergrund-Farbe der internen Komponenten.
	 */

	public Radio(String name, String[] items, CB cb, int id, boolean rand, Color fg, Color bg, Color sfg, Color sbg) {

		super(name, rand ? RAND : NONE, fg, bg);

		this.cb = cb;
		this.id = id;

		rbox   = new CheckboxGroup();
		cboxes = new Checkbox[items.length];

		for (int i=0; i < items.length; i++) {

			cboxes[i] = new Checkbox(items[i], false, rbox);

			cboxes[i].setForeground(sfg == null ? FG : sfg);
			cboxes[i].setBackground(sbg == null ? BG : sbg);

			if (cb != null) cboxes[i].addItemListener(this);

			addBag(cboxes[i],
				0, i, 1, 1, W,
				i==0 ? PAD : 0,
				i == items.length - 1 ? PAD : 0,
				PAD, PAD);
		}

		rbox.setSelectedCheckbox(cboxes[0]);
	}

	/** Erzeugt ein Radio-Objekt mit Default-Farben. */

	public Radio(String name, String[] items, CB cb, int id, boolean rand) {

		this(name, items, cb, id, rand, null, null, null, null);
	}

	/** Erzeugt ein Radio-Objekt mit Rand. */

	public Radio(String name, String[] items, CB cb, int id, Color fg, Color bg, Color sfg, Color sbg) {

		this(name, items, cb, id, true, fg, bg, sfg, sbg);
	}

	/** Erzeugt ein Radio-Objekt mit Rand und Default-Farben. */

	public Radio(String name, String[] items, CB cb, int id) {

		this(name, items, cb, id, true, null, null, null, null);
	}

	/** Erzeugt ein Radio-Objekt mit Rand und Default-Farben ohne Callback. */

	public Radio(String name, String[] items) {

		this(name, items, null, 0, true, null, null, null, null);
	}

	/**
	 * EventListener-Methode.
	 *
 	 * Der zugehoerige Callback wird aufgerufen.
	 * Argument: Ausgewaehltes Item.
	 */

	public void itemStateChanged(ItemEvent e) {

		cb.callback(id, this, rbox.getSelectedCheckbox().getLabel());
	}

	/**
	 * Setzt das aktive Item.
	 *
	 * @param item Neues aktives Item.
	 */

	public void setActiveItem(String item) {

		for (int i=0; i < cboxes.length; i++) {

			if (item.equals(cboxes[i].getLabel())) {

				rbox.setSelectedCheckbox(cboxes[i]);

				break;
			}
		}
	}

	/**
	 * Liefert das aktive Item.
	 *
	 * @return Gibt das aktive Item als String zurueck.
	 */

	public String getActiveItem() {

		return(rbox.getSelectedCheckbox().getLabel());
	}

	/**
	 * Aendert die Farben des Radio-Objekts.
	 *
	 * @param fg   Vordergrund-Farbe der Box.
	 * @param bg   Hintergrund-Farbe der Box.
	 * @param sfg  Vordergrund-Farbe der internen Komponenten.
	 * @param sbg  Hintergrund-Farbe der internen Komponenten.
	 */

	public void chgColors(Color fg, Color bg, Color sfg, Color sbg) {

		chgBoxColors(fg, bg);

		for (int i=0; i < cboxes.length; i++) {

			cboxes[i].setForeground(sfg == null ? FG : sfg);
			cboxes[i].setBackground(sbg == null ? BG : sbg);
		}
	}
}
