
import Controls.*;
import java.awt.*;
import java.awt.image.*;
import java.io.*;
import java.lang.Math;		
import java.util.Random;		// Fuer Julia-Menge

public class Master implements CB, Ctrl {

	public Frame vater;			// Parent Frame

	/* MALFELD */

	// WIDTH muss >= HEIGHT sein!
	public final static int WIDTH  = 600;	// Groesse des Malfelds
	public final static int HEIGHT = 500;
	public double ratio = (double)WIDTH/(double)HEIGHT;	// Verhaeltnis der Seiten

        boolean[][] pixel_map = new boolean[WIDTH][HEIGHT];	// gesetzte Punkte im Malfeld
	private Graphics G, g;			// Grafikkontext des Malfelds

	/* TARGETS = Komponenten der Applikation */

	private MalFeld  malFeld;		// Malfeld
	private Regler   iterRegler;		// Regler fuer die Anzahl der Iterationen
	private Schalter farbigSchalter;	// Schalter fuer farbige Mandelbrotmengen
	private Ausgabe  msgAusgabe;		// Text-Ausgabe
	

	/* CALLBACKS */

	public final static int CB_QUIT		= -1;
	public final static int CB_INIT		= 0;
	public final static int CB_CLEAR	= 1;
	public final static int CB_MAUS		= 2;
	public final static int CB_KOCH	        = 3;
	public final static int CB_BAUM	    	= 4;
	public final static int CB_LSYS	    	= 5;
	public final static int CB_LSYS_EINGABE	= 6;
	public final static int CB_MANDS_MOD	= 9;
	public final static int CB_MANDS	= 10;
	public final static int CB_MANDS_NEW	= 11;
	public final static int CB_FENSTER	= 12;
	public final static int CB_JULIA_RUECK	= 13;
	public final static int CB_JULIA_RAND	= 14;
	public final static int CB_JULIA_VOLL	= 15;
	public final static int CB_KOMPLEX	= 16;
	public final static int CB_FARBIG	= 17;
	public final static int CB_ITER		= 19;


	/* FLAGS */

	private final static int FAKTOR_INIT	= 1;
	public final static int ITER_INIT	= 100;
	public final static int ITER_MIN	= 1;
	public final static int ITER_MAX	= 1024;
	private final static int FENSTER_SAMMEL	= 9;
	private final static int FENSTER_MOTION	= 10;
	private final static int FENSTER_FINAL	= 11;
	private final static int SHOW_DITHER	= 12;
	private final static int KOMPLEX	= 13;

	private final static int NOTHING 	= 29;
	private final static int KOCH		= 30;
	private final static int BAUM		= 31;
	private final static int MANDS		= 32;
	private final static int JULIA_VOLL	= 33;
	private final static int JULIA_RAND	= 34;
	private final static int JULIA_RUECK	= 35;
	private final static int JULIA      	= 36;
	private final static int LSYS      	= 37;
	private final static int LSYS_EINGABE  	= 38;
	

	// fuer koch_stern()
	public  static final double sin_60   = -0.8660254;
	public  static final double cos_60   =  0.5000000;


	// fuer baum()
	private int anzahl =0;
	private Point[] ansatz = new Point[MAX_POINTS];



	//fuer mandel()
	Komplex	start = new Komplex();	/* linke obere Ecke der komplexen Flaeche		*/
	Komplex stop  = new Komplex();	/* rechte untere Ecke der kompl. Flaeche */
	Komplex neustart = new Komplex();	/* linke obere Ecke des aktuellen Fensters*/
	double	schritt ;		/* Schrittweite		 */
	int	max_iter = ITER_INIT; 		/* vorgegebene Iterationszahl (muss gerade sein)	*/
	private int NUM_COL;		// Anzahl der verschiedenen Farben
					// max. Anzahl Farben
	private final static int MAX_COL = 1024;
					
	int 	fall = 1; 			/* gemaess Fall fall	 */

	// Fuer julia()
	Komplex c;			// fuer diesen Punkt e C wird die
					// Julia-Menge berechnet
	private final static int FIRST_POINT	= 50; /* fuer julia_rueck	*/

	// fuer l-system
	private char w[] = new char[MAX_POINTS]; // wort mit Beschreibung der Kurve
	private int delta;              // Abstand in X- bzw. Y-Richtung
	private int regellaenge = 8;
	private char regel[]={'r','u','r','d','d','r','u','r','u','l','u','r','r','u','l','u','l','d','l','u','u','l','d','l','d','r','d','l','l','d','r','d'};

/*******************************************************************/
/* Ende der fuer die Fraktal-Applikation eingefuegte Konstanten    */
/* und Variablen						   */
/*******************************************************************/

	/* GLOBAL */

	private boolean farbig  = false;	// Flag fuer farbige Mandelbrotmenge

	private Color farbe[] = new Color[MAX_COL];	// Array mit Farben

	private int flag	= NOTHING;	// Flag fuer endl. Automaten

	private int faktor = FAKTOR_INIT;

	private int s_index, vor, nach;		// Indices im Polygon

	/* CLIPPING */

	private Point fenster_ursprung = new Point();	// Fensterursprung
	private Point fenster_delta    = new Point(2,2);	// Fensterweite

        private int xmin, xmax, ymin, ymax;		// Fensterraender

        private static final byte EMPTY  = 0;
        private static final byte LEFT   = 1;
        private static final byte RIGHT  = 2;		// Bereichscodes
        private static final byte BOTTOM = 4;
        private static final byte TOP    = 8;

	/* PUNKTE */

	private Point mom;
	private Point alt;

	private static final int MAX_POINTS = 40000;

        private int poly_point_cnt = 0;
        private Point[] poly_punkt = new Point[MAX_POINTS];

	// fuer zeitaufwendige Routinen
	/* CURSOR */

	Cursor normal_cursor = new Cursor(Cursor.DEFAULT_CURSOR);
	Cursor wait_cursor   = new Cursor(Cursor.WAIT_CURSOR);

/****** INITIALIZE *****************************/

        public void setTargets(MalFeld mf, Regler iteration, Schalter farbig, Ausgabe msg) {

                malFeld         = mf;
		iterRegler	= iteration;
		farbigSchalter	= farbig;
		msgAusgabe      = msg;
	}

        private void initialize() {

		int lauf1, lauf2, lauf3;
		int index = 0;

                g = malFeld.getGBuffer();	// Grafikkontext des Hintergrundspeichers
                G = malFeld.getGScreen();	// Grafikkontext des Malfelds
	

		/*
		// 216 Standard-Farben; poppig
		for(lauf1 = 0; lauf1 < 256; lauf1 +=51)
		{
		    for(lauf2 = 0; lauf2 < 256; lauf2 +=51)
		    {
			for(lauf3 = 0; lauf3 < 256; lauf3 += 51)
			{
			    //msgAusgabe.meldung("index: "+ index);
			    if(index < MAX_COL)
			    {
			        farbe[index++] = new Color(lauf1, lauf2, lauf3);
			    }
			}
		    }
		}
		
		NUM_COL = 216;
		*/

		// Regenbogen mit 48 FARBEN
		for(lauf1 =0; lauf1 < 256; lauf1+= 51)
		{
			farbe[index++] = new Color(lauf1, 0, 0);
		}

		for(lauf1 =0; lauf1 < 256; lauf1+= 51)
		{
			farbe[index++] = new Color(255, lauf1, 0);
		}

		for(lauf1 =255; lauf1 >= 0; lauf1-= 51)
		{
			farbe[index++] = new Color(lauf1, 255, 0);
		}

		for(lauf1 =0; lauf1 < 256; lauf1+= 51)
		{
			farbe[index++] = new Color(0, 255, lauf1);
		}

		for(lauf1 =255; lauf1 >= 0; lauf1-= 51)
		{
			farbe[index++] = new Color(0, lauf1, 255);
		}

		for(lauf1 =0; lauf1 < 256; lauf1+= 51)
		{
			farbe[index++] = new Color(lauf1, 0, 255);
		}

		for(lauf1 =0; lauf1 < 256; lauf1+= 51)
		{
			farbe[index++] = new Color(255, lauf1, 255);
		}

		for(lauf1 =255; lauf1 >= 0; lauf1-= 51)
		{
			farbe[index++] = new Color(lauf1, lauf1, lauf1);
		}
		NUM_COL = 48;

		/*
		// Regenbogen mit 1024 FARBEN
		for(lauf1 =0; lauf1 < 255; lauf1+= 2)
		{
			farbe[index++] = new Color(lauf1, 0, 0);
		}

		for(lauf1 =0; lauf1 < 255; lauf1+= 2)
		{
			farbe[index++] = new Color(255, lauf1, 0);
		}

		for(lauf1 =255; lauf1 >= 0; lauf1-= 2)
		{
			farbe[index++] = new Color(lauf1, 255, 0);
		}

		for(lauf1 =0; lauf1 < 255; lauf1+= 2)
		{
			farbe[index++] = new Color(0, 255, lauf1);
		}

		for(lauf1 =255; lauf1 >= 0; lauf1-= 2)
		{
			farbe[index++] = new Color(0, lauf1, 255);
		}

		for(lauf1 =0; lauf1 < 255; lauf1+= 2)
		{
			farbe[index++] = new Color(lauf1, 0, 255);
		}

		for(lauf1 =0; lauf1 < 255; lauf1+= 2)
		{
			farbe[index++] = new Color(255, lauf1, 255);
		}

		for(lauf1 =255; lauf1 >= 0; lauf1-= 2)
		{
			farbe[index++] = new Color(lauf1, lauf1, lauf1);
		}
		NUM_COL = 1024;
		*/
}
/***********************************************/
/******* CALLBACKS *****************************/
/***********************************************/

	public void callback(int id, Object caller, Object params) {

		switch (id) {

			case CB_QUIT: System.exit(0);
				break;

			case CB_INIT: initialize();
				break;

			case CB_CLEAR: clear();
				break;

			case CB_MAUS: maus((MalFeld) caller, (Maus) params);
				break;

			case CB_ITER: max_iter = ((Integer)params).intValue();
				break;

			case CB_FARBIG: farbig = ((Boolean) params).booleanValue();
				//msgAusgabe.meldung("farbig: "+ farbig);
				break;

			case CB_KOCH: koch_aufruf();
				break;

			case CB_BAUM: baum_aufruf();
				break;

			case CB_LSYS: lsystem();
				break;

			case CB_LSYS_EINGABE: translate_rule((String)params);
				break;

			case CB_MANDS_NEW:	
				flag = NOTHING;

			case CB_MANDS: 
				fall = 1;	// ohne Modulo rechnen
				mands_aufruf();
				break;

			case CB_MANDS_MOD: 
				fall = 2;	// Modulo rechnen
				mands_aufruf();
				break;

			case CB_FENSTER: 
				// Neues Fenster nur zulassen, wenn entweder unmittelbar
				// zuvor die Mandelbrotmenge (oder ein Teil davon)
				// gezeichnet wurde oder ein Fenster aufgezogen wurde
				// oder ein Punkt c e C gewaehlt wurde
				if((flag == MANDS) || 
				   (flag == FENSTER_FINAL) ||
				   (flag == KOMPLEX))
				{
					flag = FENSTER_SAMMEL;
				}
				break;

			case CB_KOMPLEX:
				// Wahl eines Punktes aus der Ebene C nur dann
				// zulassen, wenn gerade ein Fenster aufgezogen
				// oder ein Teil der Mandelbrotmenge gezeichnet
				// wurde
				if(flag == FENSTER_FINAL)
				{
					// Fenster loeschen
					set_window(fenster_ursprung, fenster_delta);
					
					// Flagge setzen
					flag = KOMPLEX;
				}
				if (flag == MANDS)
				{
					flag = KOMPLEX;
				}
				break;
					
			case CB_JULIA_RUECK:
				if((flag == KOMPLEX) || (flag == JULIA))
				{
					julia_aufruf(JULIA_RUECK);
				}
				break;
 
			case CB_JULIA_RAND:
				if((flag == KOMPLEX) || (flag == JULIA))
				{
					julia_aufruf(JULIA_RAND);
				}
				break;
 
			case CB_JULIA_VOLL:
				if((flag == KOMPLEX) || (flag == JULIA))
				{
					julia_aufruf(JULIA_VOLL);
				}
				break;

			default:
				msgAusgabe.meldung("Error! Wrong ID: " + id);
		}
	}

/****** Maus-Events im Malfeld *************************/

	public void maus(MalFeld canvas, Maus arg) {

                mom = normier(point(arg.loc.x, arg.loc.y));

		switch (arg.typ) {

			case Maus.MOVE:  mouseMoved(arg);
				break;

			case Maus.PRESS: mousePressed(arg);
				break;

			case Maus.RELEASE: mouseReleased(arg);
				break;

			case Maus.DRAG: mouseDragged(arg);
				break;
		}
	}

/****** Maus-Callbacks ********************************/
		
	private void mousePressed(Maus arg) {

                switch (flag) {


			case FENSTER_FINAL:
                        case FENSTER_SAMMEL:
                                fenster_ursprung.x = mom.x;
                                fenster_ursprung.y = mom.y;
                                fenster_delta.x    = 0;
                                fenster_delta.y    = 0;

                                flag = FENSTER_MOTION;
                                break;

			case KOCH:
			case BAUM:
			case MANDS:
			case NOTHING:
			case JULIA:
				break;

			case KOMPLEX:
				// aktuelle Mausposition in Starpunkt kopieren
				c = new Komplex(mom, start, schritt);
				msgAusgabe.meldung("Punkt (Re, Im): ("+ 
				c.re +
				", " +
				c.im + ")");
				break;

                        default:
                                msgAusgabe.meldung("Unknown flag " + flag + " in mousePressed");
                }

		malFeld.showBuffer();
		alt = mom;
	}

        private void mouseMoved(Maus arg) {

                switch (flag) {
 
		default:
			break;
                }
        }

        public void mouseReleased(Maus arg) {

                switch (flag) {

                        case FENSTER_MOTION:
                                if (fenster_delta.x == 0) fenster_delta.x = 1;  // Fenster mindestens
                                if (fenster_delta.y == 0) fenster_delta.y = 1;  // Groesse 1

                                if (fenster_delta.x < 0) {		// Fenster nach links
                                        fenster_ursprung.x += fenster_delta.x;
                                        fenster_delta.x = -fenster_delta.x;
                                }

                                if (fenster_delta.y < 0) {		// Fenster nach oben
                                        fenster_ursprung.y += fenster_delta.y;
                                        fenster_delta.y = -fenster_delta.y;
                                }

				// Das Fenster ist fertig
                                flag = FENSTER_FINAL;
                                break;
		}
	}

        public void mouseDragged(Maus arg) {
 
                switch (flag) {
 
                        case FENSTER_MOTION:
                                set_window(fenster_ursprung, fenster_delta); // altes Fenster loeschen mit xor
                                fenster_delta.x = mom.x - fenster_ursprung.x;// Fenstergroessen setzen
				// Fenster wird automatisch im Verhaeltnis von Hoehe und 
				// Breite des Malfeldes aufgezogen
                                fenster_delta.y = (int)(fenster_delta.x/ratio);
                                set_window(fenster_ursprung, fenster_delta); // neues Fenster mit xor

				neustart.re = start.re + schritt*fenster_ursprung.x;
				neustart.im = start.im + schritt*fenster_ursprung.y;
				
				stop.re = neustart.re + schritt*fenster_delta.x;
				stop.im = neustart.im + schritt*fenster_delta.y;
				msgAusgabe.meldung(neustart.re+" <= Real <= "+stop.re);
				msgAusgabe.meldung(neustart.im+" <= Imag <= "+stop.im);

                                break;

			case KOMPLEX:
				// aktuelle Mausposition in Starpunkt kopieren
				c = new Komplex(mom, start, schritt);
				msgAusgabe.meldung("Punkt (Re, Im): ("+ 
				(start.re + (mom.x * schritt)) +
				", " +
				(start.im + (mom.y * schritt)) + ")");
				break;
				
		}
	}

/******************************************/
/****** DRAWING ***************************/
/******************************************/

	public void clear_screen() {

		g.clearRect(0, 0, 1000, 1000);
		malFeld.showBuffer();
	}

	private void set_pixel(Point p) {	// setzt Pixel

                set_pixel_in_map(p);		// auch im Speicher (fuer Fuellen)

                if (faktor == 1) {
                        g.drawLine(p.x * faktor, p.y * faktor, p.x * faktor, p.y * faktor);
                } else {
                        g.fillArc(p.x * faktor, p.y * faktor, faktor, faktor, 0, 360);
                }
        }

	private void set_col_pixel(Point p, Color newcolor) 
	// setzt farbiges Pixel in Farbe newcolor
	{	

		Color oldcolor = g.getColor();

		g.setColor(newcolor);

                set_pixel_in_map(p);		// auch im Speicher (fuer Fuellen)

                if (faktor == 1) {
                        g.drawLine(p.x * faktor, p.y * faktor, p.x * faktor, p.y * faktor);
                } else {
                        g.fillArc(p.x * faktor, p.y * faktor, faktor, faktor, 0, 360);
                }

		g.setColor(oldcolor);
        }

	private void del_pixel(Point p) {		// loescht Pixel auch im Speicher

		Color prev_color = g.getColor();		// Farbe merken

                g.setColor(malFeld.getFeldFarbe());		// Hintergrundfarbe setzen
		set_pixel(p);
		g.setColor(prev_color);				// Farbe zuruecksetzen
        }

        private void set_xor_pixel(Point p) {	// setzt Pixel (nicht im Speicher!)

                if (faktor == 1) {
                        G.drawLine(p.x * faktor, p.y * faktor, p.x * faktor, p.y * faktor);
                } else {
                        G.fillArc(p.x * faktor, p.y * faktor, faktor, faktor, 0, 360);
                }
        }

	private void set_line(Point p, Point q) {

                bresenham_linie(p, q);
        }

        void del_line(                                          // loescht Linie
                Point p, Point q)                               // von Position p zu Position q
        {
		Color prev_color = g.getColor();		// Farbe merken

                g.setColor(malFeld.getFeldFarbe());		// Hintergrundfarbe setzen

		set_line(p, q);

                g.setColor(prev_color);				// Farbe zuruecksetzen
        }

        void set_dash(                              // zeichnet im Xor-Modus gestrichelte Linie
                Point p, Point q)                   // von Position p nach Position q
        {
		G.setXORMode(malFeld.getFeldFarbe());	// XOR-Mode ein

                bresenham_strichel(p,q);

		G.setPaintMode();			// XOR-Mode aus
        }


        /***************** zum Zeichnen und Loeschen von gestrichelten Fenstern *****************/
 
        void set_window(                    // zeichnet im Xor-Modus ein gestricheltes Fenster
                Point p,                    // beginnend bei Position p
                Point delta)                // mit Breite delta.x und Hoehe delta.y
        {
                set_dash(point(p.x,         p.y),           point(p.x + delta.x,  p.y));
                set_dash(point(p.x+delta.x, p.y),           point(p.x + delta.x,  p.y + delta.y));
                set_dash(point(p.x,         p.y),           point(p.x,            p.y + delta.y));
                set_dash(point(p.x,         p.y + delta.y), point(p.x + delta.x,  p.y + delta.y));
        }

        /****************************************************************************************/
        /*                                                                                      */
        /*                      Bresenham-Algorithmus zum Zeichnen einer Linie                  */
        /*                                                                                      */
        /****************************************************************************************/

        private void bresenham_linie(                  /* zeichnet mit Bresenham-Algorithmus    */
                Point P, Point q)                      /* eine Linie von Punkt P nach Punkt Q   */
        {
                Point p = new Point(P);

                int error, delta, schwelle, dx, dy, inc_x, inc_y;
                dx = q.x - p.x;
                dy = q.y - p.y;

                if (dx>0) inc_x= 1; else inc_x=-1;
                if (dy>0) inc_y= 1; else inc_y=-1;

                if (Math.abs(dy) < Math.abs(dx)) {             /* flach nach oben oder flach nach unten */

                        error = -Math.abs(dx);
                        delta = 2*Math.abs(dy);
                        schwelle = 2*error;
                        while (p.x != q.x) {
                                set_pixel(p);
                                p.x+=inc_x;
                                error = error + delta;
                                if (error >0) { p.y+=inc_y; error = error + schwelle;}
                        }
                } else                                 /* steil nach oben oder steil nach unten */
                {
                        error = -Math.abs(dy);
                        delta = 2*Math.abs(dx);
                        schwelle = 2*error;
                        while (p.y != q.y) {
                                set_pixel(p);
                                p.y+=inc_y;
                                error = error + delta;
                                if (error >0) { p.x+=inc_x; error = error + schwelle;}
                        }
 
                }
                set_pixel(q);
        }

        /****************************************************************************************/
        /*                                                                                      */
        /*                      Bresenham-Algorithmus zum Zeichnen einer gestrichelten Linie    */
        /*                                                                                      */
        /****************************************************************************************/

        private void bresenham_strichel(  // zeichnet mit Bresenham-Algorithmus eine gestrichelte
                Point P, Point q)         // Linie von Punkt P nach Punkt Q im XOR-Modus
        {
                Point p = new Point(P);
 
                int error, delta, schwelle, dx, dy, inc_x, inc_y;
                int steps;
 
                steps = 0;
                dx = q.x - p.x;
                dy = q.y - p.y;
 
                if (dx>0) inc_x= 1; else inc_x=-1;
                if (dy>0) inc_y= 1; else inc_y=-1;
 
                if (Math.abs(dy) < Math.abs(dx)) {              // flach nach oben oder flach nach unten
                        
                        error = -Math.abs(dx);
                        delta = 2*Math.abs(dy);
                        schwelle = 2*error;
                        while (p.x != q.x) {
                                if (((steps++ / 3) % 2 ) == 0 ) set_xor_pixel(p);
                                p.x+=inc_x;
                                error = error + delta;
                                if (error >0) { p.y+=inc_y; error = error + schwelle;}
                        }
                } else                                  // steil nach oben oder steil nach unten
                {
                        error = -Math.abs(dy);
                        delta = 2*Math.abs(dx);
                        schwelle = 2*error;
                        while (p.y != q.y) {
                                if (((steps++ / 3) % 2 ) == 0 ) set_xor_pixel(p);
                                p.y+=inc_y;
                                error = error + delta;
                                if (error >0) { p.x+=inc_x; error = error + schwelle;}
                        }
 
                }
                if (((steps++ / 3) % 2 ) == 0 ) set_xor_pixel(q);
        }

        private void set_polygon() { // zeichnet ein Polygon
                                     // anhand der Punkte-Liste poly_punkt
                                     // mit poly_point_cnt Punkten
		malFeld.setCursor(wait_cursor);
                for (int i=0; i<poly_point_cnt; i++)
		{
                        set_line(poly_punkt[i],poly_punkt[(i+1)%poly_point_cnt]);
		//	msgAusgabe.meldung("#: "+i+"; x: "+poly_punkt[i].x+
		//			   "; y: "+poly_punkt[i].y);
		}
		malFeld.setCursor(normal_cursor);
        }

/**********************************************/
/****** UTILITIES *****************************/
/**********************************************/

	private void clear_objects() {

		poly_point_cnt  = 0;	// Punkte-Zaehler zuruecksetzen
	}

	private void reset_widgets() {
		int lauf;

		// Ausgabe loeschen
		for(lauf = 0; lauf < 7; lauf ++)
		{
			msgAusgabe.meldung("");
		}

		// Zunaechst soll nichts passieren
		flag = NOTHING;			

		//faktorRegler.setValue(FAKTOR_INIT);
		faktor = FAKTOR_INIT;		// Vergroesserungsfaktor

		//farbigSchalter.setValue(false);
		// farbig = false;

		// Initiale Anzahl iterationen setzen
		max_iter = ITER_INIT;
                iterRegler.setValue(max_iter);

		// Auswahl-Fenster auf ganzen Bildschirm setzen
		fenster_ursprung.x = 0;
		fenster_ursprung.y = 0;
		fenster_delta.x    = WIDTH;
		fenster_delta.y    = HEIGHT;
	}

	private void clear() {

		clear_screen();
		clear_map();
		clear_objects();
		reset_widgets();
	}

        private Point normier(Point p) {	// normiert die Koordinaten eines Punktes

                Point q = new Point();

		q.x = p.x / faktor;
		q.y = p.y / faktor;

                return q;
        }

        private void set_pixel_in_map(Point p) {	// setzt Pixel im Speicher (fuer Fuellen)
 
                if (!range_ok(p)) return; 		// ausserhalb
 
		if (g.getColor().equals(malFeld.getFeldFarbe()))	// wenn Hintergrundfarbe
			pixel_map[p.x][p.y] = false;			//   loeschen
		else							// sonst
			pixel_map[p.x][p.y] = true;			//   setzen
        }

        private boolean get_pixel(Point p) {		// Ist Punkt p gesetzt?

                return pixel_map[p.x][p.y];
        }

        private void clear_map() {

		malFeld.setCursor(wait_cursor);
                for (int x = 0; x < WIDTH; x++)
                        for (int y = 0; y < HEIGHT; y++)
                                pixel_map[x][y] = false;
		malFeld.setCursor(normal_cursor);
        }

        private void show_map() {

		malFeld.setCursor(wait_cursor);
                for (int x = 0; x < WIDTH/faktor; x++)
                        for (int y = 0; y < HEIGHT/faktor; y++)
                                if (pixel_map[x][y]) set_pixel(point(x, y));
		malFeld.setCursor(normal_cursor);

		malFeld.showBuffer();
        }

        private boolean range_ok(Point p) {

                return (((0<=p.x) && (0<=p.y)) &&
                       ((WIDTH > p.x * faktor) &&
                       (HEIGHT > p.y * faktor)));
        }

        private Point point(int x, int y) {
 
                Point p = new Point(x, y);

                return p;
        }

/****************************************************************************************/
/*
/*		Neue Methoden fuer die Fraktal-Applikation
/*
/****************************************************************************************/


	public void koch_aufruf()
	// beim ersten Aufruf von koch_aufruf() wird die Ausgangsfigur gezeichnet
	// bei jedem weiteren Aufruf wird eine Iteration von koch_stern aufgerufen
	{
	    // FALLS Button KOCH 1-tes Mal gedrueckt wurde
	    if( flag != KOCH)
	    {
		clear_screen(); 
		clear_map(); 
		clear_objects(); 

		// Uebernehmen der Button-Id
		flag = KOCH;

		// Dreiecksseiten
		double a, b, c;
		
		c = WIDTH/2;
		a = c/2;
		b = WIDTH*Math.sqrt(12)/8;
		
		double bereich = 6*HEIGHT/5;

		// Start-Dreieck zeichnen
		poly_punkt[0] = new Point((int)(0.5*(WIDTH - c)),(int)(0.5*(bereich - b)));
		poly_punkt[1] = new Point((int)(0.5*(WIDTH + c)),(int)(0.5*(bereich - b)));
		poly_punkt[2] = new Point((int)(WIDTH/2)        ,(int)(0.5*(bereich + b)));


		// Zahl der Eckpunkte setzen
		poly_point_cnt = 3;

		// Linien zwischen allen Eckpunkten zeichnen
		set_polygon();
	    }
	    // SONST (d.h. KOCH wurde unmittelbar zuvor
	    // schonmal gedrueckt): naechste Iteration
	    else
	    {
		// FALLS noch eine Iteration moeglich ist
		if(poly_point_cnt < MAX_POINTS/4)
		{
			// Loeschen
			clear_map();
			clear_screen(); 
			poly_point_cnt = koch_stern(poly_punkt, poly_point_cnt);
			// Linien zwischen allen Eckpunkten zeichnen
			set_polygon();
		}
		else
		{
			msgAusgabe.meldung("Maximale Anzahl Polygonpunkte erreicht");
		}
	    }
	    malFeld.showBuffer();
	    msgAusgabe.meldung("Anzahl der Polygonpunkte: " + poly_point_cnt);
	}

/*****************************************************************************/
/*                                                                             */
/*                        Koch'sche Schneeflocke                                     */
/*                                                                             */
/*****************************************************************************/

private int koch_stern(                // wendet eine Koch-Iteration an auf
        Point poly_punkt[],     // das Polygon poly_punkt
        int poly_punkt_cnt)     // mit poly_point_cnt Punkten
                                // neue Anzahl Punkte wird zurueckgeliefert
{
	malFeld.setCursor(wait_cursor);

        int i,j;
        double x0,x1,x2,x3,x4,y0,y1,y2,y3,y4;
        Point[] HPunkt = new Point[MAX_POINTS];
        j=0;

        for (i=0; i<poly_point_cnt; i++)
        {
                x0 = poly_punkt[i].x;
                y0 = poly_punkt[i].y;
                
                x4 = poly_punkt[(i+1)%(poly_point_cnt)].x;
                y4 = poly_punkt[(i+1)%(poly_point_cnt)].y;

                x1 = (x4-x0)/3+x0;
                y1 = (y4-y0)/3+y0;
                        
                x3 = x4 - (x4-x0)/3;
                y3 = y4 - (y4-y0)/3;

                x2 = (int)(x1+ (x3-x1)*cos_60 - (y3-y1)*sin_60);
                y2 = (int)(y1+ (y3-y1)*cos_60 + (x3-x1)*sin_60);

                HPunkt[j]   = new Point();
                HPunkt[j+1]   = new Point();
                HPunkt[j+2]   = new Point();
                HPunkt[j+3]   = new Point();

                HPunkt[j]        = poly_punkt[i];
                HPunkt[j+1].x        = (int)x1;
                HPunkt[j+1].y        = (int)y1;
                HPunkt[j+2].x        = (int)x2;
                HPunkt[j+2].y        = (int)y2;
                HPunkt[j+3].x        = (int)x3;
                HPunkt[j+3].y        = (int)y3;
                j=j+4;
        }

        for (i=0; i<j; i++)                      // neu berechnetes Polygon 
                poly_punkt[i] = HPunkt[i];       // kopieren

	malFeld.setCursor(normal_cursor);

        return j;                                // neue Anzahl Punkte zurueck
}

public void baum_aufruf()
// beim ersten Aufruf von baum_aufruf() wird die Ausgangsfigur gezeichnet
// bei jedem weiteren ein Iteration von baum() aufgerufen
{
    // FALLS Button BAUM 1-tes Mal gedrueckt wurde
	    if( flag != BAUM)
	    {
		clear_screen(); 
		clear_map(); 
		clear_objects(); 

		// Uebernehmen der Button-Id
		flag = BAUM;

		// Start-Linie definieren
		poly_punkt[0] = new Point((int)(4*WIDTH/7 - WIDTH/10), (int)(0.95*HEIGHT));
		poly_punkt[1] = new Point((int)(4*WIDTH/7), (int)(0.95*HEIGHT));

		//msgAusgabe.meldung("p0: ("+poly_punkt[0].x+", "+poly_punkt[0].y+")");
		//msgAusgabe.meldung("p1: ("+poly_punkt[1].x+", "+poly_punkt[1].y+")");

		// Zahl der Eckpunkte setzen
		poly_point_cnt = 2;

		// Start-Ansaetze definieren
		ansatz[0] = new Point(poly_punkt[0]);
		ansatz[1] = new Point(poly_punkt[1]);

		// Zahl der Ansatzstuecke setzen
		anzahl =2;

		// Linien zwischen allen Eckpunkten zeichnen
		set_polygon();

	    }
	    // SONST (d.h. BAUM wurde unmittelbar zuvor
	    // schonmal gedrueckt): naechste Iteration
	    else
	    {
		// FALLS noch eine Iteration moeglich ist
		if(2*poly_point_cnt + 1 < MAX_POINTS)
		{
			baum();
		}
		else
		{
			msgAusgabe.meldung("Maximale Anzahl Polygonpunkte erreicht!");
		}
	    }
	    malFeld.showBuffer();
	    msgAusgabe.meldung("Anzahl Polygonpunkte: " + poly_point_cnt);
	}


/*****************************************************************************/
/*                                                                           */
/*                              Baum                                         */
/*                                                                           */
/*****************************************************************************/

public void baum()             // wendet eine Iteration auf die Ansatzstuecke an
        //Point ansatz[],      // alte und neue Folge von Ansatzstuecken
        //int   anzahl,        // Anzahl der Anzsatzstueckendpunkte
        //Point poly_punkt[],  // alte und neue Folge von Polygonecken
        //int   poly_point_cnt // Anzahl der Polygonecken
{
	malFeld.setCursor(wait_cursor);

        int i,j,k,p;
        double a,b,l,si_de,co_de;
        Point P;
        Point Q;
        Point A;
        Point B;
        Point C;
        Point[] ast = new Point[3];
        Point[] Hpunkt = new Point[MAX_POINTS];
        Point[] Hansatz = new Point[MAX_POINTS];

        DoublePoint[] schema = new DoublePoint[3];

        ast[0] = new Point();
        ast[1] = new Point();
        ast[2] = new Point();
        
        schema[0] = new DoublePoint();
        schema[1] = new DoublePoint();
        schema[2] = new DoublePoint();

        schema[0].x = 0.0;                    // generiere Schema
        schema[0].y = 2.0;                                
        schema[1].x = 2.0/3.0;
        schema[1].y = 7.0/3.0;
        schema[2].x = 1.0;
        schema[2].y = 2.0;

        j=0;                                  // Index fuer neue Ansatzstuecke
        k=0;                                  // Index fuer neue Polygonpunkte
        for (p=0; p<anzahl; p=p+2)
        {
                P = ansatz[p];
                Q = ansatz[p+1];
                
                a     = P.y - Q.y;            // bestimme l, sinus, cosinus
                b     = Q.x - P.x;
                l     = Math.sqrt(a*a + b*b);
                si_de = a/l;
                co_de = b/l;

                for (i=0; i<3; i++)           // fuehre Skalierung und Rotation aus
                {
                    ast[i].x = P.x + (int)(l*schema[i].x*co_de-l*schema[i].y*si_de);
                    ast[i].y = P.y - (int)(l*schema[i].y*co_de+l*schema[i].x*si_de);
                }

                A = ast[0];                   // kreiere neue Ast-Punkte
                B = ast[1];
                C = ast[2];

                if (anzahl>2) del_line(P, Q); // loesche Ansatzstueck

                set_line(P,A);                // zeichne Figur P,A,B,C,Q
                set_line(A,B);
                set_line(B,C);
                set_line(C,Q);
                                              // speichere neue Ansatzstuecke
                Hansatz[j]          = new Point(A); 
                Hansatz[j+1]        = new Point(B);
                Hansatz[j+2]        = new Point(B);
                Hansatz[j+3]        = new Point(C);
                j=j+4;
        }

        i=0;                                  // Index fuer altes Polygon
        k=0;                                  // Index fuer neues Polygon
        for (p=0; p<anzahl; p=p+2)            // Index fuer alte Ansatzstuecke
        {
                while  ((poly_punkt[i].x !=ansatz[p].x) ||
                        (poly_punkt[i].y !=ansatz[p].y))
                                              // uebernimm unveraenderte Polygonpkte
                    Hpunkt[k++] = poly_punkt[i++];
 
                Hpunkt[k]   = ansatz[p];      // uebernimm neue Polygonpunke
                Hpunkt[k+1] = Hansatz[2*p];
                Hpunkt[k+2] = Hansatz[2*p+1];
                Hpunkt[k+3] = Hansatz[2*p+3];
                Hpunkt[k+4] = ansatz[p+1];
                k+=4;
                i++;
        }

        while (i<poly_point_cnt) Hpunkt[k++]=poly_punkt[i++];
        
        for (i=0; i<j; i++) 
                ansatz[i] = Hansatz[i];       // neu berechnete Ansatzstuecke kopieren 
        anzahl=j;                             
        
        for (i=0; i<k; i++) 
                poly_punkt[i] = Hpunkt[i];    // neu berechnete Polygonpunkte kopieren
        poly_point_cnt=k;                   

	malFeld.setCursor(normal_cursor);
}


public void mands_aufruf()
// beim Aufruf von mands_aufruf() wird der -  durch das vorher gesetzte
// Fenster - markierte Bereich von C dargestellt.
{
            malFeld.setCursor(wait_cursor);
	    // FALLS unmittelbar zuvor "Mand s" oder "Mand s%" gedrueckt wurde
	    // einfach mit aktuellem Fenster nochmal rechnen
	    if(flag == MANDS)
	    {
                    //clear_screen();
		    //clear_map();
	    }
	    // SONST (d.h.: unmittelbar zuvor wurde ein Fenster aufgezogen)
	    else if(flag == FENSTER_FINAL) 
	    {
		// altes Bild wegwerfen
		clear_screen();
		//clear_map();
	    
		// neue Koordinaten berechnen
		start.re += schritt*fenster_ursprung.x;
		start.im += schritt*fenster_ursprung.y;
		//msgAusgabe.meldung("fenster_delta: ("+fenster_delta.x + ", "+fenster_delta.y+")");
		schritt  = (schritt*fenster_delta.x) / WIDTH;
		//msgAusgabe.meldung("schritt: "+ schritt);
	    }
	    // FALLS Button MANDS 1-tes Mal gedrueckt wurde
	    // oder Button MANDS % 1-tes Mal gedrueckt wurde
	    else
	    {
		// Alles wegwerfen
		clear_screen(); 
		//clear_map(); 
		clear_objects(); 

		// Automatisch skalieren, so dass die ganze Mandelbrotmenge sichtbar ist
		if(WIDTH >= HEIGHT)
		{
			// Hier ist die beste Startecke
			start.im = -1.40;
			schritt = 2.8 / HEIGHT;
			start.re = -0.7 + ((-WIDTH/2) * schritt);
		}
		else
		{
			// Hier ist die beste Startecke
			start.re = -2.15;
			schritt = 2.9 / WIDTH;
			start.im = (-HEIGHT/2) * schritt;
		}

	    }
	    
	    // feste parameter
	    if((farbig == true) && (fall == 1))
	    {
		fall = 3;
	    }
	    else if ((farbig == true) && (fall == 2))
	    {
	    	fall = 4;
	    }

	    // zeichnen
	    mandel(start, schritt, max_iter, fall);

	    // zeigen
	    malFeld.showBuffer();

	    // aktuellen Ausschnitt angeben
	    msgAusgabe.meldung(max_iter+" Iterationen");
	    stop.re = start.re + schritt*WIDTH;
	    stop.im = start.im + schritt*HEIGHT;
	    msgAusgabe.meldung(start.re+" <= Real <= "+stop.re);
	    msgAusgabe.meldung(start.im+" <= Imag <= "+stop.im);

	    // Es wurde gerade "Mand s" oder "Mand s%" gedrueckt
	    flag = MANDS;

            malFeld.setCursor(normal_cursor);
	}	


/*****************************************************************************/
/*                    Mandelbrotmenge                                        */
/*****************************************************************************/
void  mandel(                   // berechnet und zeichnet Mandelbrotmenge ab 
        Komplex start,          // linker oberer Ecke start der komplexen Flaeche
        double  schritt,        // mit Schrittweite schritt und 
        int     max_iter,       // Iterationsanzahl max_iter (muss gerade sein)
        int     fall)           // gemaess Fall fall
                                // Abbruch erfolgt, wenn der Betrag groesser als 2 ist 
                                // oder wenn Iterationszahl erreicht ist 
{
    Point p = new Point();
    int zaehler ;
    Komplex c;
    Komplex z;
    double betrag,tmp;

    for (p.x = 0; p.x < WIDTH; p.x++)
    {
    for (p.y = 0; p.y < HEIGHT;  p.y++)
    {
        c = new Komplex(p, start, schritt);
        z = new Komplex(0.0, 0.0);
        zaehler = 0;
        betrag  = 0.0;

        while( (betrag < 4.0)  && (zaehler++ < max_iter) ) {
                tmp    = z.re;
                z.re   = z.re * z.re - z.im * z.im + c.re;
                z.im   = 2 * tmp * z.im + c.im;
                betrag = z.re*z.re + z.im*z.im;
        }
                
        switch (fall) {
        case 1:                                // schwarz/weiss
            if (betrag < 4.0) 
            {
                set_pixel(p);                  // schwarz
            }
            else 
            {
                del_pixel(p);                  // weiss
            }
            break; 
 
        case 2:                                // schwarz/weiss mit modulo
            if (betrag < 4.0)                  // Mandelbrotmenge selbst
            {
                set_pixel(p);                  // schwarz
            }
            else                               // Bereich um Mandelbrot
            {
                if ((zaehler %2) != 0)         // modulo
                {
                    set_pixel(p);              // schwarz
                }
                else
                {
                    del_pixel(p);              // weiss
                }
            }
            break;

        case 3:                               // farbig
            if(zaehler == max_iter+1)
            {
                set_col_pixel(p,farbe[0]);    // Mandelbrotmenge selbst
            }
            else                              // Bereich um Mandelbrot
            {
                set_col_pixel(p,farbe[zaehler%NUM_COL]);  // farbe modulo
            }
            break;

        case 4:                               // farbig mit modulo
            if(zaehler == max_iter+1)
            {
                set_col_pixel(p,farbe[0]);    // Mandelbrotmenge selbst
            }
 	    else                              // Bereich um Mandelbrot
            {
                 if((zaehler % 2) != 0)       // farbe modulo modulo
                 {
                     set_col_pixel(p,farbe[zaehler%NUM_COL]);  
                 }
                 else
                 {
                     del_pixel(p);            // weiss
                 }
            }
            break;
        }
    }
    malFeld.showBuffer();                        // Nach jeder Spalte darstellen
    }
}


public void julia_aufruf(int mode)
// ruft je nach mode eine der Julia-Zeichen-Methoden auf
{
	malFeld.setCursor(wait_cursor);
	Komplex julia_start = new Komplex();
	// Hier ist die beste Startecke
	julia_start.re = ((-WIDTH/2) * schritt);
	julia_start.im = (-HEIGHT/2) * schritt;

        if(flag == KOMPLEX)
	{
	        //clear_map();
        	clear_screen();
		flag = JULIA;
	}

	switch(mode)
	{
		case JULIA_VOLL:
			julia_voll(c, julia_start, schritt, max_iter);
			msgAusgabe.meldung(max_iter+" Iterationen");
			break;
		
		case JULIA_RAND:
			julia_rand(c, julia_start, schritt, max_iter);
			msgAusgabe.meldung(max_iter+" Iterationen");
			break;
		
		case JULIA_RUECK:
        		clear_screen();
			// Anzahl der Rueckschritte 5000
			julia_rueck(c, julia_start, schritt, 5000);
			break;
		
		default: 
			msgAusgabe.meldung("julia_aufruf: unknown mode: "+ mode);
	}

	// aktuellen Punkt aus C angeben:
	msgAusgabe.meldung("c = "+c.re+" + "+c.im);

	// aktuellen Ausschnitt angeben
	stop.re = julia_start.re + schritt*WIDTH;
	stop.im = julia_start.im + schritt*HEIGHT;
	msgAusgabe.meldung(julia_start.re+" <= Real <= "+stop.re);
	msgAusgabe.meldung(julia_start.im+" <= Imag <= "+stop.im);

	malFeld.setCursor(normal_cursor);
	malFeld.showBuffer();
}



/*************************************************************************/
/*                                                                       */
/*                               Juliamenge                              */
/*                                                                       */
/*************************************************************************/


static Komplex f(Komplex z, Komplex c)                        
// berechnet eine Iteration der Transformation
// z = z*z + c
{
        double tmp;
        
        tmp  = z.re;
        z.re = z.re * z.re - z.im * z.im + c.re;
        z.im = 2 * tmp * z.im + c.im;
        return (z);
}


static double q_betrag(       // liefert das Quadrat des Betrags
        Komplex z)            // der komplexen Zahl z
{
        return (z.re * z.re + z.im* z.im);
}


void  julia_voll(         // berechnet Julia-Menge als gefuelltes Gebiet
        Komplex c,        // fuer komplexes c, 
        Komplex start,    // die linke obere Ecke start der komplexen Flaeche,
        double schritt,   // Schrittweite schritt  
        int max_iter)     // mit vorgegebener Iterationszahl max_iter
{
        Point p = new Point();
        Point q = new Point();          // Symmetrischer Punkt
        int zaehler ;
        Komplex z;
 
        for (p.x = 0; p.x <= WIDTH; p.x++)
        {                               // Wegen Symmetrie nur bis zur Haelfte laufen
        for (p.y = 0; p.y <= HEIGHT/2;  p.y++)
        {
                z = new Komplex(p, start, schritt);

                q.x = WIDTH  - p.x;     // Symmetrie nutzen
                q.y = HEIGHT - p.y;

                zaehler = 0;
                while ((q_betrag(z) < 4.0) && (zaehler++ < max_iter))
                {
                        z = f(z,c);
                }
        
                if(!farbig)
                {
                        if (q_betrag(z) < 4.0)  
                        {
                                set_pixel(p); 
                                set_pixel(q);
                        }
                        else 
                        {
                                del_pixel(p);
                                del_pixel(q);
                        }
                }
                else
                {
                        set_col_pixel(p, farbe[zaehler%max_iter]);
                        set_col_pixel(q, farbe[zaehler%max_iter]);
                }

        }
        malFeld.showBuffer();
        }
}


void  julia_rand(       // berechnet Julia-Menge als Umrandung
       Komplex c,       // fuer komplexes c, 
       Komplex start,   // die linke obere Ecke start der komplexen Flaeche,
       double schritt,  // Schrittweite schritt 
       int max_iter)    // mit vorgegebener Iterationszahl max_iter
{
        Point p = new Point();
        Point q = new Point();
        int zaehler ;
        Komplex z;
        boolean draussen;
 
        for (p.x = 0; p.x <= WIDTH; p.x++) {
            draussen = true;
            for (p.y = 0; p.y <= (int)(HEIGHT/2 + 0.5) ;  p.y++)
            {                         // Wegen Symmetrie nur bis zur Haelfte laufen
                q.x = WIDTH  - p.x;   // Symmetrie nutzen
                q.y = HEIGHT - p.y;

                z       = new Komplex(p,start, schritt);
                zaehler = 0;
                while ((q_betrag(z) < 4.0) && (zaehler++ < max_iter)) 
                {
                        z = f(z,c);
                }
        
                if (q_betrag(z) < 4.0)                   // hat konvergiert
                {
                    if (draussen)                        // falls draussen
                    {
                            set_pixel(p);                // male schwarz
                            set_pixel(q);                // male schwarz
                            draussen = false;            // nun drinnen
                    } 
                    else 
                    {
                        del_pixel(p);                    // male weiss
                        del_pixel(q);                    // male weiss
                    }
                } 
                else                                     // hat nicht konvergiert
                {
                        if (!draussen)                   // falls drinnen
                        {
                            set_pixel(point(p.x,p.y-1)); // male Vorgaenger schwarz
                            set_pixel(point(q.x,q.y+1)); // male Vorgaenger schwarz
                            draussen = true;             // nun draussen
                        } 
                        else 
                        {
                            del_pixel(p);                // male weiss
                            del_pixel(q);                // male weiss
                        }
                }
            }
            malFeld.showBuffer();
        }
}


public Komplex complex_square_root(       // berechnet die Wurzel (d.h.  r mit r*r=z)
        Komplex z)                        // zur komplexen Zahl z
{
        double  betrag;
        Komplex result = new Komplex();
        
        betrag = Math.sqrt(z.re * z.re + z.im * z.im);

        if ((betrag+z.re)< 0.0) 
                result.re = 0.0; 
        else 
                result.re = Math.sqrt((betrag+z.re)/2.0);

        if ((betrag-z.re)< 0.0) 
                result.im = 0.0; 
        else 
                result.im = Math.sqrt((betrag-z.re)/2.0);

        if (z.im < 0.0)         
                result.im = -result.im;

        return(result);
}

public Komplex backward_random(       // fuehrt einen Rueckwaertsschritt aus
                Komplex z,            // angewendet auf die kompexe Zahl z
                Komplex c)            // und die komplexe Zahl c. Liefert ihn
                                      // mit zufaelligem Vorzeichen ab
{
        Komplex result = new Komplex();

        result.re = z.re - c.re;
        result.im = z.im - c.im;
        result    = complex_square_root(result);

        if (Math.random() < 0.5)
        {
                result.re=-result.re;
                result.im=-result.im;
        }

        return(result);
}

void julia_rueck(          // berechnet Julia-Menge
        Komplex c,         // fuer komplexes c
        Komplex start,     // linke obere Ecke der komplexen Flaeche
        double schritt,    // Schrittweite        
        int anzahl)        // durch anzahl Rueckwaertsspruenge
{
        int k;
        Komplex z = new Komplex(1.0, 0.0);   // Startposition

        for (k=0; k<FIRST_POINT; k++)        // tastet sich an die Julia-Kurve heran
            z = backward_random(z,c);


        for (k=0; k < anzahl; k++) {                         
            z = backward_random(z,c);        // bestimme Anzahl Vorgaenger
            set_pixel(z.getPoint(start, schritt)); // und zeichne sie
            if(k%20 == 0)
            {
                malFeld.showBuffer();
            }
        }

}

public void lsystem()
// startet die naechste Iteration des Lindenmayersystems
{
	Point m;            // Mittelpunkt des Bildschirms
	int halbd;	    // Haelfte des Abstandes
	int lauf;

	clear_screen();

	if(flag != LSYS)
	{
		flag = LSYS;              // Flagge setzen

		for(lauf =0; lauf < MAX_POINTS; lauf++)
		{
			w[lauf] ='\0';
		}
		
		w[0] = 'r';               // initiales Quadrat
		w[1] = 'u';
		w[2] = 'l';
		w[3] = 'd';


	        m = new Point((int)(WIDTH/2), (int)(HEIGHT/2));
		delta = (int)(WIDTH/3);
	        halbd = (int)(delta/2);

		poly_punkt[0] = new Point(m.x - halbd, m.y + halbd);
		poly_punkt[1] = new Point(m.x + halbd, m.y + halbd);
		poly_punkt[2] = new Point(m.x + halbd, m.y - halbd);
		poly_punkt[3] = new Point(m.x - halbd, m.y - halbd);
		poly_punkt[4] = new Point(m.x - halbd, m.y + halbd);

		poly_point_cnt = 5;       // Anzahl setzen

	}
	else
	{
		if(poly_point_cnt*regellaenge < MAX_POINTS)
		{
			delta = (int)(2*delta/regellaenge);
			lsystem_iteration(regellaenge, regel, delta);
		}
	}
	msgAusgabe.meldung("Das Wort w: "+w); // ausgeben
	msgAusgabe.meldung(""); 
	msgAusgabe.meldung("Anzahl Polygonpunkte: "+ poly_point_cnt+"\n");
	if(poly_point_cnt*regellaenge >= MAX_POINTS)
	{
		msgAusgabe.meldung("Maximale Anzahl Polygonpunkte"+
			 	   " erreicht");
	}
		
	set_polygon();            // darstellen
	malFeld.showBuffer();

}

public void lsystem_iteration(  // fuehrt eine Iteration des Lindenmayer-
                                // Systems durch. 
        int regellaenge,        // Die Laenge der einzelnen Regeln
        char[] regel,                // Die Regel, enthaelt hintereinander die
                                // Regeln fuer 'r', 'u', 'l', 'd' 
        int delta)              // Abstand zwischen zwei Punkten in X- und 
                                // Y-Richtung
        //char[] w,               // Das Wort
        //Point[] poly_punkt,     // Die Punkte
        //int poly_point_cnt,     // Die Anzahl der Elemente im Wort
{
        int lauf;
        int lauf2;
        int index =0;
        int regelnr = 0;
        char w_neu[] = new char[MAX_POINTS];
        Point[] poly_neu= new Point[MAX_POINTS];

        malFeld.setCursor(wait_cursor);
	
        for(lauf =0; lauf < poly_point_cnt - 1; lauf++)
        {
            index = regellaenge * lauf;

            // Erster neuer Punkt identisch mit Altem
            poly_neu[index++] = new Point(poly_punkt[lauf]);

            switch (w[lauf])
                {
                    case 'r':
                        regelnr = 0;
                        break;
                case 'u':
                        regelnr = 1;
                        break;
                case 'l': 
                        regelnr = 2;
                        break;
                case 'd':
                        regelnr = 3;
                        break;
                default:
                        msgAusgabe.meldung("Unknown regel >>>"+
                        w[lauf] + "<<< in lsystem_iteration\n");
                        break;
            }

            // Laut regel die regellaenge Stueck neuen Punkte erstellen
            for(lauf2 = 0; lauf2 < regellaenge; lauf2 ++)
            {
                // letzten Punkt kopieren
                poly_neu[index] = new Point(poly_neu[index-1]);

                // neues Wort-Element erzeugen
                w_neu[index-1] = regel[regelnr*regellaenge + lauf2];

                // aktuelles Element der Regel finden
                // und Punkt an die neue Position verschieben
                switch(regel[regelnr*regellaenge + lauf2])
                {
                    case 'r':
                        poly_neu[index].x += delta;
                        break;
                    case 'u':
                        poly_neu[index].y -= delta;
                        break;
                    case 'l':
                        poly_neu[index].x -= delta;
                        break;
                    case 'd':
                        poly_neu[index].y += delta;
                        break;
                    default:
                        msgAusgabe.meldung("Unknown regel >>>"+
                                regel[regelnr*regellaenge+lauf2]+"<<<\n");
                        break;
                }
                index++;
            }
        }

        for(lauf =0; lauf <= index; lauf++)
        {
                poly_punkt[lauf] = poly_neu[lauf];
                if(lauf != index)
                {
                        w[lauf] = w_neu[lauf];
                }
        }
        poly_point_cnt = index;

        malFeld.setCursor(wait_cursor);
                        
}

public void translate_rule(String rule)
// uebersetzt die eingegebene Regel und schreibt den Inhalt in regel
{
	int lauf;
	int lauf2;
	String r = new String("r");
	String l = new String("l");
	String d = new String("d");
	String u = new String("u");
	
	//msgAusgabe.meldung("rule: "+rule);
	
	flag = LSYS_EINGABE;

	regellaenge = rule.length();      // reale Laenge holen

	for(lauf =0; lauf < regellaenge; lauf ++)
	{
	    rule=new String(rule.concat(rule.valueOf(rule.charAt(regellaenge-lauf-1))));
	}

	regellaenge *=2;
 
	// Regel fuer 'u' konstruieren
	for(lauf =0; lauf < regellaenge; lauf ++)
	{
	    if(rule.regionMatches(lauf, r, 0, 1))
    	    {
		rule=new String(rule.concat(u));
	    }
    	    else if(rule.regionMatches(lauf, u, 0, 1))
	    {
	        rule=new String(rule.concat(l));
	    }
	    else if(rule.regionMatches(lauf, l, 0, 1))
	    {
	        rule=new String(rule.concat(d));
	    }
	    else if(rule.regionMatches(lauf, d, 0, 1))
	    {
	        rule=new String(rule.concat(r));
	    }
	}

	// Regel fuer 'l' konstruieren
	for(lauf =0; lauf < regellaenge; lauf ++)
	{
	    if(rule.regionMatches(lauf, r, 0, 1))
    	    {
		rule=new String(rule.concat(l));
	    }
    	    else if(rule.regionMatches(lauf, u, 0, 1))
	    {
	        rule=new String(rule.concat(d));
	    }
	    else if(rule.regionMatches(lauf, l, 0, 1))
	    {
	        rule=new String(rule.concat(r));
	    }
	    else if(rule.regionMatches(lauf, d, 0, 1))
	    {
	        rule=new String(rule.concat(u));
	    }
	}

	// Regel fuer 'd' konstruieren
	for(lauf =0; lauf < regellaenge; lauf ++)
	{
	    if(rule.regionMatches(lauf, r, 0, 1))
    	    {
		rule=new String(rule.concat(d));
	    }
    	    else if(rule.regionMatches(lauf, u, 0, 1))
	    {
	        rule=new String(rule.concat(r));
	    }
	    else if(rule.regionMatches(lauf, l, 0, 1))
	    {
	        rule=new String(rule.concat(u));
	    }
	    else if(rule.regionMatches(lauf, d, 0, 1))
	    {
	        rule=new String(rule.concat(l));
	    }
	}
	
	regel = rule.toCharArray();
	lsystem();
}


public void setParent(Frame f)
// setzt den Parent Frame auf f
{
	vater = f;
}





}



