
import Controls.*;
import java.awt.*;

public class Master implements CB, Ctrl {

	/* MALFELD */

	public final static int WIDTH  = 1000;	// Groesse des Malfelds
	public final static int HEIGHT = 1000;
        boolean[][] pixel_map = new boolean[WIDTH][HEIGHT];	// gesetzte Punkte im Malfeld
	private Graphics G, g;			// Grafikkontext des Malfelds

	/* TARGETS = Komponenten der Applikation */

	private MalFeld  malFeld;		// Malfeld
	private Auswahl  functionAuswahl; 	// Choice
	private Regler   faktorRegler; 		// Slider
	private Regler   ditherRegler;		// Slider

	/* CALLBACKS */

	public final static int CB_QUIT		= -1;
	public final static int CB_INIT		= 0;
	public final static int CB_CLEAR	= 1;
	public final static int CB_FUNCTION	= 2;
	public final static int CB_MAUS		= 3;
	public final static int CB_FAKTOR	= 4;
	public final static int CB_DITHER	= 5;

	/* FLAGS */

	private final static int PUNKT		= 0;
	private final static int LINIE		= 1;
	private final static int KREIS		= 2;
	private final static int VOLLKREIS	= 3;
	private final static int POLYGON_START	= 4;
	private final static int POLYGON_SAMMEL	= 5;
	private final static int POLYGON_FINDE	= 6;
	private final static int POLYGON_MOTION	= 7;
	private final static int FENSTER_START	= 8;
	private final static int FENSTER_MOTION	= 9;

	/* SLIDER-CONST */

        public  static final int FAKTOR_INIT = 1;
        public  static final int FAKTOR_MIN  = 1;
        public  static final int FAKTOR_MAX = 20;

	/* DITHERING */

	private static final int DITHER_DIM  = 8;
        public  static final int DITHER_MAX = DITHER_DIM * DITHER_DIM;
        public  static final int DITHER_INIT = DITHER_MAX / 2;

        private static final int[][] dit_mat =
        {       { 0,32, 8,40, 2,34,10,42},                      /* 64-elementige Dithermatrix:  */
                {48,16,56,24,50,18,58,26},                      /* Bei Grauwert k werden alle   */
                {12,44, 4,36,14,46, 6,38},                      /* Pixel i,j auf schwarz        */
                {60,28,52,20,62,30,54,22},                      /* gesetzt, fuer die gilt:      */
                { 3,35,11,43, 1,33, 9,41},                      /* dit_mat[i%8,j%8] < k         */
                {51,19,59,27,49,17,57,25},
                {15,47, 7,39,13,45, 5,37},
                {63,31,55,23,61,29,53,21},
        };

	/* GLOBAL */

	private int flag = PUNKT;		// Flag fuer endl. Automaten
	private boolean first;			// Flag fuer Linie + Kreis

	private int faktor = FAKTOR_INIT;	// Vergroesserungsfaktor

	private int schwelle = DITHER_INIT;	// Dither-Schwellwert

	private int s_index, vor, nach;		// Indices im Polygon

	/* CLIPPING */

	private Point fenster_ursprung = new Point();	// Fensterursprung
	private Point fenster_delta    = new Point();	// Fensterweite

	/* PUNKTE */

	private Point mom;
	private Point p1;
	private Point alt;

	private static final int MAX_POINTS = 1000;

        private int poly_point_cnt = 0;
        private Point[] poly_punkt = new Point[MAX_POINTS];

	/* CURSOR */

	Cursor normal_cursor = new Cursor(Cursor.DEFAULT_CURSOR);
	Cursor wait_cursor   = new Cursor(Cursor.WAIT_CURSOR);

/****** INITIALIZE *****************************/

        public void setTargets(MalFeld mf, Auswahl func, Regler fak, Regler dith) {

                malFeld         = mf;
		functionAuswahl = func;
		faktorRegler    = fak;
		ditherRegler    = dith;
	}

        private void initialize() {

                g = malFeld.getGBuffer();	// Grafikkontext des Hintergrundspeichers
                G = malFeld.getGScreen();	// Grafikkontext des Malfelds
        }

/***********************************************/
/****** CALLBACK *******************************/
/***********************************************/

	public void callback(int id, Object caller, Object params) {

		switch (id) {

			case CB_QUIT: System.exit(0);
				break;

			case CB_INIT: initialize();
				break;

			case CB_CLEAR: clear();
				break;

			case CB_FUNCTION: function((String) params);
				break;

			case CB_MAUS: maus((MalFeld) caller, (Maus) params);
				break;

			case CB_FAKTOR: set_faktor(((Integer) params).intValue());
				break;

			case CB_DITHER: schwelle = ((Integer) params).intValue();
				break;

			default:
				System.err.println("Error! Wrong ID: " + id);
		}
	}

	private void set_faktor(int val) {

		faktor = val;

		clear_screen();

		show_map();	// zeigt die gespeicherten Punkte
	}

/****** Object Choice ***************************/

	public void function(String item) {

		reset_polygon();

		if (item.equals("Punkt")) {
			flag = PUNKT;

		} else if (item.equals("Linie")) {
			flag = LINIE;
			first = true;

		} else if (item.equals("Kreis")) {
			flag = KREIS;
			first = true;

		} else if (item.equals("Voll-Kreis")) {
			flag = VOLLKREIS;
			first = true;

		} else if (item.equals("Polygon")) {
			flag = POLYGON_START;

		} else if (item.equals("Clip-Fenster")) {
			flag = FENSTER_START;

		} else {
			System.err.println("Choice-error: " + item);
		}
	}

/****** Maus-Events im Malfeld *************************/

	public void maus(MalFeld canvas, Maus arg) {

                mom = normier(new Point(arg.loc.x, arg.loc.y));

		switch (arg.typ) {

			case Maus.MOVE:  mouseMoved(arg);
				break;

			case Maus.PRESS: mousePressed(arg);
				break;

			case Maus.RELEASE: mouseReleased(arg);
				break;

			case Maus.DRAG: mouseDragged(arg);
				break;

			default:	// ignore other mouse events
		}
	}

/****** Maus-Callbacks ********************************/
		
	private void mousePressed(Maus arg) {

                switch (flag) {

                        case PUNKT:
                                set_pixel(mom);		// Punkt zeichnen
                                break;

			case LINIE:
				if (first) {		// erster Punkt

					p1 = mom;		// Punkt merken
					set_pixel(mom);		// Eckpunkt zeichnen

				} else {		// zweiter Punkt

					bresenham_linie(p1,mom); // Linie zeichnen
				}

				first = !first;
				break;

			case KREIS:
				if (first) {		// erster Punkt

					p1 = mom;		// Punkt merken
					set_pixel(mom);		// Mittelpunkt zeichnen

				} else {		// zweiter Punkt

 					bresenham_kreis(p1,radius(p1, mom)); // Kreis zeichnen
				}

                                first = !first;
                                break;

			case VOLLKREIS:
				if (first) {		// erster Punkt

					p1 = mom;		// Punkt merken
					set_pixel(mom);		// Mittelpunkt zeichnen

				} else {		// zweiter Punkt

 					bresenham_kreis(p1,radius(p1, mom)); // Kreis zeichnen
 					bresenham_kreis_fill(p1,radius(p1, mom)); // Kreis fuellen
				}

                                first = !first;
                                break;

                        case POLYGON_START:
                                poly_punkt[poly_point_cnt=0] = mom;	// Punkt speichern
                                set_marker(mom);			// Eckregion markieren
                                poly_point_cnt++;
                                flag = POLYGON_SAMMEL;
                                break;
 
                        case POLYGON_SAMMEL:
                                if (check_region(poly_punkt[0],mom)) {			// Polygon ist fertig
                                  set_dash(poly_punkt[poly_point_cnt-1],alt);		// loesche letztes Gummiband
                                  set_line(poly_punkt[poly_point_cnt-1],poly_punkt[0]);	// zeichne Linie zum 1. Knoten

                                  flag=POLYGON_FINDE;                                                        

                                } else {					// weiterer Punkt gefunden
                                  poly_punkt[poly_point_cnt] = mom;		// speichern
                                  set_dash(poly_punkt[poly_point_cnt-1],alt);	// loesche letztes Gummiband
                                  set_line(poly_punkt[poly_point_cnt-1],mom);	// zeichne Linie zum neuen Knoten
                                  set_marker(mom);				// Eckregion markieren
                                  poly_point_cnt++;
                                }
				malFeld.showBuffer();
                                break;

                        case POLYGON_FINDE:

                                boolean gefunden = false;
 
                                for (int i=0; (i<poly_point_cnt) && (!gefunden); i++)	// suche den Polygonpunkt,
                                if (check_region(poly_punkt[i],mom))			// auf den geklickt wurde
                                {
                                        gefunden = true;				// gefunden!
                                        mom=poly_punkt[i];				// wird akt. Punkt
                                        s_index = i;					// merke Index

                                        if (s_index==0) vor=poly_point_cnt-1; else vor=s_index-1;	// Vorgaenger
                                        if (s_index==(poly_point_cnt-1)) nach=0; else nach=s_index+1;	// Nachfolger
 
                                        del_line(poly_punkt[vor],mom);		// loesche beide Kanten
                                        del_line(mom, poly_punkt[nach]);	// zu Vorgaener und Nachfolger

                                        del_marker(mom);			// loesche Marker

					malFeld.showBuffer();
 
                                        set_dash(poly_punkt[vor],mom);		// setze gestrichelte Linien
                                        set_dash(mom, poly_punkt[nach]);	// zu Vorgaener und Nachfolger

                                        flag=POLYGON_MOTION;
                                }
                                break;
		
                        case FENSTER_START:
                                fenster_ursprung.x = mom.x;
                                fenster_ursprung.y = mom.y;
                                fenster_delta.x    = 0;
                                fenster_delta.y    = 0;

                                flag = FENSTER_MOTION;
                                break;

                        default:
                                System.err.println("Unknown flag " + flag + " in mousePressed");
                }

		if ((flag != POLYGON_FINDE)&&(flag != POLYGON_SAMMEL)&&(flag != POLYGON_MOTION)) malFeld.showBuffer();
		alt = mom;
	}

        private void mouseMoved(Maus arg) {

                switch (flag) {
 
                        case POLYGON_SAMMEL:
                                set_dash(poly_punkt[poly_point_cnt-1], alt);	// loesche Gummiband
                                set_dash(poly_punkt[poly_point_cnt-1], mom);	// zeichne neues Gummiband
				alt = mom;
 
                                break;
                }
        }

        public void mouseReleased(Maus arg) {

                switch (flag) {

                        case POLYGON_MOTION:
                                poly_punkt[s_index] = mom;        // speichere momentane Koordinaten ab
 
                                set_dash(poly_punkt[vor],alt);    // loesche beide Gummibaender
                                set_dash(alt,poly_punkt[nach]);
 
                                set_line(poly_punkt[vor],mom);    // zeichne solide Kanten
                                set_line(mom, poly_punkt[nach]);

                                set_marker(mom);                  // zeichne Marker

                                flag=POLYGON_FINDE;

				malFeld.showBuffer();

                                break;

                        case FENSTER_MOTION:
                                if (fenster_delta.x == 0) fenster_delta.x = 1;  // Fenster mindestens
                                if (fenster_delta.y == 0) fenster_delta.y = 1;  // Groesse 1

                                if (fenster_delta.x < 0) {		// Fenster nach links
                                        fenster_ursprung.x += fenster_delta.x;
                                        fenster_delta.x = -fenster_delta.x;
                                }

                                if (fenster_delta.y < 0) {		// Fenster nach oben
                                        fenster_ursprung.y += fenster_delta.y;
                                        fenster_delta.y = -fenster_delta.y;
                                }

				clipping();	// Mit dem aufgezogenen Fenster wird geclippt

                                flag = FENSTER_START;
                                break;
		}
	}

        public void mouseDragged(Maus arg) {
 
                switch (flag) {
 
                        case POLYGON_MOTION:
 
                                set_dash(poly_punkt[vor], alt);           // loesche beide Gummibaender
                                set_dash(alt, poly_punkt[nach]);
 
                                set_dash(poly_punkt[vor], mom);           // zeichne neue Gummibaender
                                set_dash(mom, poly_punkt[nach]);
 
                                alt = mom;                                // merke alte Position
                                break;

                        case FENSTER_MOTION:
                                set_window(fenster_ursprung, fenster_delta); // altes Fenster loeschen mit xor
                                fenster_delta.x = mom.x - fenster_ursprung.x;// Fenstergroessen setzen
                                fenster_delta.y = mom.y - fenster_ursprung.y;
                                set_window(fenster_ursprung, fenster_delta); // neues Fenster mit xor
                                break;
		}
	}

/******************************************/
/****** DRAWING ***************************/
/******************************************/

	public void clear_screen() {

		g.clearRect(0, 0, WIDTH, HEIGHT);
		malFeld.showBuffer();
	}

	private void set_pixel(Point p) {	// setzt Pixel

                set_pixel_in_map(p);		// auch im Speicher

                if (faktor == 1) {
                        g.drawLine(p.x * faktor, p.y * faktor, p.x * faktor, p.y * faktor);
                } else {
                        g.fillArc(p.x * faktor, p.y * faktor, faktor, faktor, 0, 360);
                }
        }

        private void set_xor_pixel(Point p) {	// setzt Pixel (nicht im Speicher!)

                if (faktor == 1) {
                        G.drawLine(p.x * faktor, p.y * faktor, p.x * faktor, p.y * faktor);
                } else {
                        G.fillArc(p.x * faktor, p.y * faktor, faktor, faktor, 0, 360);
                }
        }

	private void set_line(Point p, Point q) {

                bresenham_linie(p, q);
        }

        void del_line(                                          // loescht Linie
                Point p, Point q)                               // von Position p zu Position q
        {
		Color prev_color = g.getColor();		// Farbe merken

                g.setColor(malFeld.getFeldFarbe());		// Hintergrundfarbe setzen

		set_line(p, q);

                g.setColor(prev_color);				// Farbe zuruecksetzen
        }

        void set_dither_line(               // zeichnet waagerechte Linie gemaess Dither-Matrix
                Point P, int x)             // von Punkt p nach Punkt (x/p.y)
        {
                Point p = new Point(P);

                while (p.x++ < x)
                        if (kleiner_schwelle(p)) set_pixel(p);
        }

        void set_dash(                              // zeichnet im Xor-Modus gestrichelte Linie
                Point p, Point q)                   // von Position p nach Position q
        {
		G.setXORMode(malFeld.getFeldFarbe());	// XOR-Mode ein

                bresenham_strichel(p,q);

		G.setPaintMode();			// XOR-Mode aus
        }

        void set_marker(		// setzt Quadrat
                Point p)		// an Position p
        {
                int delta;

                g.setPaintMode();

                if (faktor==1) {
                        g.drawRect(p.x-1,p.y-1,2,2);
                } else {
                        delta=faktor/2;
                        g.drawRect(faktor*p.x,faktor*p.y,2*delta,2*delta);
                }
        }

        void del_marker(                                        // loescht Quadrat
                Point p)                                        // an Position p
        {
		Color prev_color = g.getColor();		// Farbe merken

                g.setColor(malFeld.getFeldFarbe());		// Hintergrundfarbe setzen

                set_marker(p);

                g.setColor(prev_color);				// Farbe zuruecksetzen
        }

        /***************** zum Zeichnen und Loeschen von gestrichelten Fenstern *****************/
 
        void set_window(                    // zeichnet im Xor-Modus ein gestricheltes Fenster
                Point p,                    // beginnend bei Position p
                Point delta)                // mit Breite delta.x und Hoehe delta.y
        {
                set_dash(new Point(p.x, p.y),           new Point(p.x + delta.x,  p.y));
                set_dash(new Point(p.x+delta.x, p.y),   new Point(p.x + delta.x,  p.y + delta.y));
                set_dash(new Point(p.x, p.y),           new Point(p.x,            p.y + delta.y));
                set_dash(new Point(p.x, p.y + delta.y), new Point(p.x + delta.x,  p.y + delta.y));
        }

        /****************************************************************************************/
        /*                                                                                      */
        /*                      Bresenham-Algorithmus zum Zeichnen einer Linie                  */
        /*                                                                                      */
        /****************************************************************************************/

        private void bresenham_linie(                  /* zeichnet mit Bresenham-Algorithmus    */
                Point P, Point q)                      /* eine Linie von Punkt P nach Punkt q   */
        {
                Point p = new Point(P);

                int error, delta, schwelle, dx, dy, inc_x, inc_y;
                dx = q.x - p.x;
                dy = q.y - p.y;

                if (dx>0) inc_x= 1; else inc_x=-1;
                if (dy>0) inc_y= 1; else inc_y=-1;

                if (Math.abs(dy) < Math.abs(dx)) {             /* flach nach oben oder flach nach unten */

                        error = -Math.abs(dx);
                        delta = 2*Math.abs(dy);
                        schwelle = 2*error;
                        while (p.x != q.x) {
                                set_pixel(p);
                                p.x+=inc_x;
                                error = error + delta;
                                if (error >0) { p.y+=inc_y; error = error + schwelle;}
                        }
                } else                                 /* steil nach oben oder steil nach unten */
                {
                        error = -Math.abs(dy);
                        delta = 2*Math.abs(dx);
                        schwelle = 2*error;
                        while (p.y != q.y) {
                                set_pixel(p);
                                p.y+=inc_y;
                                error = error + delta;
                                if (error >0) { p.x+=inc_x; error = error + schwelle;}
                        }
 
                }
                set_pixel(q);
        }

        /****************************************************************************************/
        /*                                                                                      */
        /*                      Bresenham-Algorithmus zum Zeichnen einer gestrichelten Linie    */
        /*                                                                                      */
        /****************************************************************************************/

        private void bresenham_strichel(  // zeichnet mit Bresenham-Algorithmus eine gestrichelte
                Point P, Point q)         // Linie von Punkt P nach Punkt Q im XOR-Modus
        {
                Point p = new Point(P);
 
                int error, delta, schwelle, dx, dy, inc_x, inc_y;
                int steps;
 
                steps = 0;
                dx = q.x - p.x;
                dy = q.y - p.y;
 
                if (dx>0) inc_x= 1; else inc_x=-1;
                if (dy>0) inc_y= 1; else inc_y=-1;
 
                if (Math.abs(dy) < Math.abs(dx)) {              // flach nach oben oder flach nach unten
                        
                        error = -Math.abs(dx);
                        delta = 2*Math.abs(dy);
                        schwelle = 2*error;
                        while (p.x != q.x) {
                                if (((steps++ / 3) % 2 ) == 0 ) set_xor_pixel(p);
                                p.x+=inc_x;
                                error = error + delta;
                                if (error >0) { p.y+=inc_y; error = error + schwelle;}
                        }
                } else                                  // steil nach oben oder steil nach unten
                {
                        error = -Math.abs(dy);
                        delta = 2*Math.abs(dx);
                        schwelle = 2*error;
                        while (p.y != q.y) {
                                if (((steps++ / 3) % 2 ) == 0 ) set_xor_pixel(p);
                                p.y+=inc_y;
                                error = error + delta;
                                if (error >0) { p.x+=inc_x; error = error + schwelle;}
                        }
 
                }
                if (((steps++ / 3) % 2 ) == 0 ) set_xor_pixel(q);
        }

        /****************************************************************************************/
        /*                                                                                      */
        /*                      Bresenham-Algorithmus zum Zeichnen eines Kreises                */
        /*                                                                                      */
        /****************************************************************************************/

        private void bresenham_kreis (                  /* zeichnet mit Bresenham-Algorithmus   */
                Point p,                                /* einen Kreis um den Punkt p           */
                int r)                                  /* mit Radius r                         */
        {
                int x,y,d,dx,dxy;
                x=0; y=r; d=1-r;
                dx=3; dxy=-2*r+5;
                while (y>=x)
                {
                        set_pixel( new Point( p.x+x, p.y+y) );  // alle 8 Oktanden werden 
                        set_pixel( new Point( p.x+y, p.y+x) );  // gleichzeitig gezeichnet
                        set_pixel( new Point( p.x+y, p.y-x) );
                        set_pixel( new Point( p.x+x, p.y-y) );
                        set_pixel( new Point( p.x-x, p.y-y) );
                        set_pixel( new Point( p.x-y, p.y-x) );
                        set_pixel( new Point( p.x-y, p.y+x) );
                        set_pixel( new Point( p.x-x, p.y+y) );

                        if (d<0) { d=d+dx;  dx=dx+2; dxy=dxy+2; x++;      }
                            else { d=d+dxy; dx=dx+2; dxy=dxy+4; x++; y--; }
                }
        }

        private void set_polygon() { // zeichnet ein Polygon
                                     // anhand der Punkte-Liste poly_punkt
                                     // mit poly_point_cnt Punkten
                for (int i=0; i<poly_point_cnt; i++)
                        set_line(poly_punkt[i],poly_punkt[(i+1)%poly_point_cnt]);
        }

	private void reset_polygon() {

                for (int i=0; i<poly_point_cnt; i++)
			del_marker(poly_punkt[i]);

		if (poly_point_cnt > 2)
			set_polygon();

		if (poly_point_cnt == 2) {
			del_line(poly_punkt[0], poly_punkt[1]);
			poly_point_cnt = 0;
		}

		malFeld.showBuffer();
	}

/**********************************************/
/****** UTILITIES *****************************/
/**********************************************/

	private void clear_objects() {

		poly_point_cnt  = 0;	// Punkte-Zaehler zuruecksetzen
	}

	private void reset_widgets() {

		functionAuswahl.setActiveItem("Punkt");
		flag = PUNKT;			// aktuelles Objekt

		faktorRegler.setValue(FAKTOR_INIT);
		faktor = FAKTOR_INIT;		// Vergroesserungsfaktor

		ditherRegler.setValue(DITHER_INIT);
		schwelle = DITHER_INIT;		// Dither-Schwellwert
	}

	private void clear() {

		clear_screen();
		clear_map();
		clear_objects();
		reset_widgets();
	}

        private Point normier(Point p) {	// normiert die Koordinaten eines Punktes

                Point q = new Point();

		q.x = p.x / faktor;
		q.y = p.y / faktor;

                return q;
        }

        private void set_pixel_in_map(Point p) {	// setzt Pixel im Speicher (fuer Fuellen)
 
                if (!range_ok(p)) return; 		// ausserhalb
 
		if (g.getColor().equals(malFeld.getFeldFarbe()))	// wenn Hintergrundfarbe
			pixel_map[p.x][p.y] = false;			//   loeschen
		else							// sonst
			pixel_map[p.x][p.y] = true;			//   setzen
        }

        private void clear_map() {

		malFeld.setCursor(wait_cursor);

                for (int x = 0; x < WIDTH; x++)
                        for (int y = 0; y < HEIGHT; y++)
                                pixel_map[x][y] = false;

		malFeld.setCursor(normal_cursor);
        }

        private void show_map() {

		malFeld.setCursor(wait_cursor);

                for (int x = 0; x < WIDTH/faktor; x++)
                        for (int y = 0; y < HEIGHT/faktor; y++)
                                if (pixel_map[x][y]) set_pixel(new Point(x, y));

		malFeld.setCursor(normal_cursor);

		malFeld.showBuffer();
        }

        private boolean range_ok(Point p) {

                return (((0<=p.x) && (0<=p.y)) &&
                       ((WIDTH > p.x * faktor) &&
                       (HEIGHT > p.y * faktor)));
        }

        private int radius(Point p, Point q) {
                double a, b;

                a = (double) ( (p.x-q.x) * (p.x-q.x));
                b = (double) ( (p.y-q.y) * (p.y-q.y));
                return (int) Math.sqrt( a + b );
        }

        private boolean check_region(Point region, Point p) { // testet, ob p in der Naehe von region liegt

                int delta;

                if      (faktor==1) delta = 3;
                else if (faktor==2) delta = 2;
                else                delta = 1;

                return ((Math.abs(region.x - p.x)<=delta) && (Math.abs(region.y - p.y)<=delta));
        }


/****** CLIPPING **************************************************************/

	private void clipping() {

		clear_screen();
		clear_map();

		malFeld.setCursor(wait_cursor);

		if (poly_point_cnt > 1) { // Polygon-Clipping nach Sutherland-Hodgmann

			poly_point_cnt = sutherland_hodgman(poly_point_cnt,poly_punkt,fenster_ursprung.x,0);
			poly_point_cnt = sutherland_hodgman(poly_point_cnt,poly_punkt,fenster_ursprung.y,1);
			poly_point_cnt = sutherland_hodgman(poly_point_cnt,poly_punkt,fenster_ursprung.x+fenster_delta.x,2);
			poly_point_cnt = sutherland_hodgman(poly_point_cnt,poly_punkt,fenster_ursprung.y+fenster_delta.y,3);

			set_polygon();
		}

		malFeld.setCursor(normal_cursor);

		malFeld.showBuffer();
	}

	/****************************************************************************************/
	/*											*/
	/*	          	Clippen von Polygonen an einem Fenster				*/
	/*	          	nach Sutherland-Hodgmann              				*/
	/*											*/
	/****************************************************************************************/
	
	private boolean On_Visible_Side(			/* liefert true, falls		*/
				Point P,			/* Punkt p auf der sichtbaren	*/
				int wert,			/* Seite der Kante wert liegt	*/
				int fall)			/* gemaess Fall fall		*/
	{
		switch (fall) {

			case 0: return (P.x >= wert);	/* linke  Fenster-Kante		*/
			case 1: return (P.y >= wert);	/* untere Fenster-Kante		*/
			case 2: return (P.x <= wert);	/* rechte Fenster-Kante		*/
			case 3: return (P.y <= wert);	/* obere  Fenster-Kante		*/
		}
	
		return false;
	}
	
	
	private boolean Intersection(			/* liefert true,			*/
				Point P1, Point P2,	/* falls sich die Linie P1-P2		*/
				int wert,		/* mit Clipping-Kante wert schneidet	*/
				int fall,		/* gemaess Fall fall			*/
				Point I)		/* liefert den Schnittpunkt I zurueck	*/
	{
		boolean P1_vis, P2_vis;			/* true, falls P1 bzw P2 sichtbar	*/
		double slope;				/* Steigung der Geraden P1-P2		*/
							/* y = (x-P1.x)*slope + P1.y		*/
	
		P1_vis = On_Visible_Side(P1,wert,fall);	/* P1 sichtbar ?			*/
		P2_vis = On_Visible_Side(P2,wert,fall);	/* P2 sichtbar ?			*/
	
		if ((P1_vis && P2_vis) || (!P1_vis && !P2_vis)) return (false); /* kein Schnittpkt.	*/
		else
		{
			slope = (double)(P2.y-P1.y) / (double) (P2.x-P1.x);   /* Steigung		*/
	
			if (fall%2==1) {
				I.x = (int)((wert-P1.y) / slope) + P1.x;
				I.y = wert;
			} else
			{
				I.x = wert;
				I.y = (int)((wert-P1.x) * slope) + P1.y;
			}
			return(true);
		}
	}
	
	private int sutherland_hodgman(			/* fuehrt ein Clipping durch		*/
				int num_points,		/* fuer num_points Punkte		*/
				Point[] points,		/* der Polygonliste Points		*/
				int wert,		/* bezueglich Clipping-Kante wert	*/
				int fall)		/* gemaess Fall fall			*/
	/* liefert neue Anzahl Punkte */
	{
		int   i;
		int   old_ptr = 0;
		int   new_ptr = 0;
		Point S;
		Point I;
		Point[] hpoints = new Point[MAX_POINTS];
	
		if (num_points > 2) {

			points[num_points] = points[0];
		 
			while (old_ptr < num_points) {
	
				S = points[old_ptr++];
				if (On_Visible_Side(S,wert,fall)) {
					hpoints[new_ptr++] = S;
				}
				I = new Point();
				if (Intersection(S,points[old_ptr],wert,fall,I)) {
					hpoints[new_ptr++] = I;
				}
			}
	
			for (i=0; i<new_ptr; i++) points[i]=hpoints[i];
		}
		return new_ptr;
	}

/****** DITHERING ****************************************************************/

        boolean kleiner_schwelle(                               /* testet die Dither-Matrix     */
                Point p)                                        /* bezueglich des Punktes p     */
        {
                int x, y;

                x = p.x; y = p.y;

                if (x<0) x=DITHER_DIM - (Math.abs(x)%DITHER_DIM);
                if (y<0) y=DITHER_DIM - (Math.abs(y)%DITHER_DIM);

                return (dit_mat[x%DITHER_DIM][y%DITHER_DIM] < schwelle);
        }

	/****************************************************************************************/
	/*											*/
	/*	       		Bresenham-Algorithmus zum Fuellen eines Kreises mit Scanline	*/
	/*											*/
	/****************************************************************************************/

	void bresenham_kreis_fill (                     /* fuellt mit Bresenham-Algorithmus     */
		Point p,                                /* einen Kreis um den Punkt p           */
		int r)                                  /* mit Radius r gemaess Dithering       */
	{
		int x,y,d,dx,dxy;

		x=0; y=r; d=1-r;
		dx=3; dxy=-2*r+5;
		while (y>=x)
		{
			set_dither_line( new Point( p.x-x,p.y+y), p.x+x);  // es werden 4 Linien
			set_dither_line( new Point( p.x-y,p.y+x), p.x+y);  // gleichzeitig 
			set_dither_line( new Point( p.x-y,p.y-x), p.x+y);  // gezeichnet  
			set_dither_line( new Point( p.x-x,p.y-y), p.x+x);

			if (d<0) { d=d+dx;  dx=dx+2; dxy=dxy+2; x++;      }
			    else { d=d+dxy; dx=dx+2; dxy=dxy+4; x++; y--; }
		}
	}
}
