
package Controls;

import java.awt.*;
import java.awt.event.*;

/** Ein Control-Objekt zur Ausgabe von beliebig vielen Meldungen. */

public class Ausgabe extends CtrlBox {

	private TextArea out;

	/**
	 * Erzeugt ein Ausgabe-Objekt mit Name, Groessen und Farben.
	 *
	 * @param name	Name der Box.
	 * @param rows	Anzahl Zeilen.
	 * @param cols	Anzahl Spalten.
	 * @param fg	Vordergrund-Farbe der Box.
	 * @param bg	Hintergrund-Farbe der Box.
	 * @param sfg	Vordergrund-Farbe der internen Komponenten.
	 * @param sbg	Hintergrund-Farbe der internen Komponenten.
	 */

	public Ausgabe(String name, int rows, int cols, Color fg, Color bg, Color sfg, Color sbg) {

		super(name, NONE, fg, bg);

		out = new TextArea(null, rows, cols, TextArea.SCROLLBARS_VERTICAL_ONLY);

		out.setForeground(sfg == null ? FG : sfg);
		out.setBackground(sbg == null ? BG : sbg);

		out.setEditable(false);

		addBag(out, 0, 0, 1, 1);
	}

	/** Erzeugt ein Ausgabe-Objekt mit Default-Groessen. */

	public Ausgabe(String name, Color fg, Color bg, Color sfg, Color sbg) {

		this(name, 25, 42, fg, bg, sfg, sbg);
	}

	/** Erzeugt ein Ausgabe-Objekt mit Default-Farben. */

	public Ausgabe(String name, int rows, int cols) {

		this(name, rows, cols, null, null, Color.black, Color.white);
	}

	/** Erzeugt ein Ausgabe-Objekt mit Default-Groessen und Default-Farben. */

	public Ausgabe(String name) {

		this(name, 25, 42, null, null, Color.black, Color.white);
	}

	/**
	 * Gibt eine Meldung im Ausgabe-Objekt aus.
	 *
	 * @param msg Auszugebende Meldung
	 */

	public void meldung(String msg) {

		out.append(msg + "\n");
	}

	/**
	 * Aendert die Farben des Ausgabe-Objekts.
	 *
	 * @param fg   Vordergrund-Farbe der Box.
	 * @param bg   Hintergrund-Farbe der Box.
	 * @param sfg  Vordergrund-Farbe der internen Komponenten.
	 * @param sbg  Hintergrund-Farbe der internen Komponenten.
	 */

	public void chgColors(Color fg, Color bg, Color sfg, Color sbg) {

		chgBoxColors(fg, bg);

		out.setForeground(sfg == null ? FG : sfg);
		out.setBackground(sbg == null ? BG : sbg);
	}
}
