
package Controls;

/**
 * Implementiert ein einheitliches Callback-Konzept.
 *
 * Dieses Interface wird von verschiedenen generischen Control-Objekten verwendet,
 * um in ihren EventListener-Methoden ein anderes Objekt mit der Behandlung des
 * Callbacks beauftragen zu koennen. Solche Master-Objekte muessen das CB-Interface
 * implementieren und werden dann bei der Konstruktion einer Applikation an das
 * entsprechende Control-Objekt uebergeben. Es kann beliebig viele Master-Objekte
 * geben, die jedes eine beliebige Anzahl von Callbacks behandeln koennen.
 */

public interface CB {

	/**
	 * Allgemeine Callback-Funktion.
	 *
	 * Diese Funktion muss von geeigneten Master-Objekten implementiert werden.
	 * @param id 	Eindeutige ID, um verschiedene Callbacks zu unterscheiden, die vom selben Master-Objekt behandelt werden.
	 * @param caller Das den Callback ausloesende Control-Objekt.
	 * @param args	Argumente, die an das Master-Objekt geliefert werden sollen.
	 */
	    
	public void callback(int id, Object caller, Object args);
}
