
package Controls;

import java.awt.*;
import java.awt.event.*;

/** Ein Control-Objekt zur Auswahl beliebig vieler Items aus einer festen Menge. */

public class Liste extends CtrlBox implements ItemListener {

	private static final int LEN = 5;

	private List list;

	private CB  cb;
	private int id;

	/**
	 * Erzeugt ein Liste-Objekt mit Name, Item-Liste, Callback, Rand und Farben.
	 *
	 * @param name	Name der Box.
	 * @param items	Liste der einzutragenden Items.
	 * @param cb	Master-Objekt fuer Callback (null -> kein EventListener).
	 * @param id	Master-ID fuer Callback.
	 * @param rand	Rand oder kein Rand.
	 * @param fg	Vordergrund-Farbe der Box.
	 * @param bg	Hintergrund-Farbe der Box.
	 * @param sfg	Vordergrund-Farbe der internen Komponenten.
	 * @param sbg	Hintergrund-Farbe der internen Komponenten.
	 */

	public Liste(String name, String[] items, CB cb, int id, boolean rand, Color fg, Color bg, Color sfg, Color sbg) {

		super(name, rand ? RAND : NONE, fg, bg);

		this.cb = cb;
		this.id = id;

		list = new List(items.length < LEN ? items.length : LEN, true);

		list.setForeground(sfg == null ? FG : sfg);
		list.setBackground(sbg == null ? BG : sbg);

		for (int i=0; i < items.length; i++) {

			list.addItem(items[i]);
		}

		if (cb != null) list.addItemListener(this);

		addBag(list, 0, 0, 1, 1, C, PAD, PAD, PAD, PAD);
	}

	/** Erzeugt ein Liste-Objekt mit Default-Farben. */

	public Liste(String name, String[] items, CB cb, int id, boolean rand) {

		this(name, items, cb, id, rand, null, null, null, null);
	}

	/** Erzeugt ein Liste-Objekt mit Rand. */

	public Liste(String name, String[] items, CB cb, int id, Color fg, Color bg, Color sfg, Color sbg) {

		this(name, items, cb, id, true, fg, bg, sfg, sbg);
	}

	/** Erzeugt ein Liste-Objekt mit Rand und Default-Farben. */

	public Liste(String name, String[] items, CB cb, int id) {

		this(name, items, cb, id, true, null, null, null, null);
	}

	/** Erzeugt ein Liste-Objekt mit Rand und Default-Farben ohne Callback. */

	public Liste(String name, String[] items) {

		this(name, items, null, 0, true, null, null, null, null);
	}

	/**
	 * EventListener-Methode.
	 *
 	 * Der zugehoerige Callback wird aufgerufen.
	 * Argument: Ausgewaehlte Items.
	 */

	public void itemStateChanged(ItemEvent e) {

		cb.callback(id, this, list.getSelectedItems());
	}

	/**
	 * Selektiert ein Item.
	 *
	 * @param item Zu selektierendes Item.
	 */

	public void selectItem(String item) {

		for (int i=0; i < list.getItemCount(); i++) {

			if (item.equals(list.getItem(i))) {

				list.select(i);

				break;
			}
		}
	}

	/**
	 * Deselektiert ein Item.
	 *
	 * @param item Zu deselektierendes Item.
	 */

	public void deselectItem(String item) {

		for (int i=0; i < list.getItemCount(); i++) {

			if (item.equals(list.getItem(i))) {

				list.deselect(i);

				break;
			}
		}
	}

	/**
	 * Liefert selektierte Items.
	 *
	 * @return Gibt die Liste der selektierten Items als Array von Strings zurueck.
	 */

	public String[] getSelectedItems() {

		return(list.getSelectedItems());
	}

	/**
	 * Aendert die Farben des Liste-Objekts.
	 *
	 * @param fg   Vordergrund-Farbe der Box.
	 * @param bg   Hintergrund-Farbe der Box.
	 * @param sfg  Vordergrund-Farbe der internen Komponenten.
	 * @param sbg  Hintergrund-Farbe der internen Komponenten.
	 */

	public void chgColors(Color fg, Color bg, Color sfg, Color sbg) {

		chgBoxColors(fg, bg);

		list.setForeground(sfg == null ? FG : sfg);
		list.setBackground(sbg == null ? BG : sbg);
	}
}
