
package Controls;

import java.awt.*;
import java.awt.event.*;

/** Ein Control-Objekt zur Auswahl eines binaeren Zustandes. */

public class Schalter extends CtrlBox implements ItemListener {

	private Checkbox cbox;

	private CB  cb;
	private int id;

	/**
	 * Erzeugt ein Schalter-Objekt mit Name, Zustand, Callback, Rand und Farben.
	 *
	 * @param name	Name der Box.
	 * @param init	Initialer Zustand.
	 * @param cb	Master-Objekt fuer Callback (null -> kein EventListener).
	 * @param id	Master-ID fuer Callback.
	 * @param rand	Rand oder kein Rand.
	 * @param fg	Vordergrund-Farbe der Box.
	 * @param bg	Hintergrund-Farbe der Box.
	 * @param sfg	Vordergrund-Farbe der internen Komponenten.
	 * @param sbg	Hintergrund-Farbe der internen Komponenten.
	 */

	public Schalter(String name, boolean init, CB cb, int id, boolean rand, Color fg, Color bg, Color sfg, Color sbg) {

		super(name, rand ? RAND : NONE, fg, bg);

		this.cb = cb;
		this.id = id;

		cbox = new Checkbox(name, init);

		cbox.setForeground(sfg == null ? FG : sfg);
		cbox.setBackground(sbg == null ? BG : sbg);

		if (cb != null) cbox.addItemListener(this);

		addBag(cbox, 0, 0, 1, 1);
	}

	/** Erzeugt ein Schalter-Objekt mit Default-Farben. */

	public Schalter(String name, boolean init, CB cb, int id, boolean rand) {

		this(name, init, cb, id, rand, null, null, null, null);
	}

	/** Erzeugt ein Schalter-Objekt ohne Rand. */

	public Schalter(String name, boolean init, CB cb, int id, Color fg, Color bg, Color sfg, Color sbg) {

		this(name, init, cb, id, false, fg, bg, sfg, sbg);
	}

	/** Erzeugt ein Schalter-Objekt ohne Rand mit Default-Farben. */

	public Schalter(String name, boolean init, CB cb, int id) {

		this(name, init, cb, id, false, null, null, null, null);
	}

	/** Erzeugt ein Schalter-Objekt ohne Rand mit Default-Farben ohne Callback. */

	public Schalter(String name, boolean init) {

		this(name, init, null, 0, false, null, null, null, null);
	}

	/**
	 * EventListener-Methode.
	 *
 	 * Der zugehoerige Callback wird aufgerufen.
	 * Argument: Ausgewaehlter Zustand.
	 */

	public void itemStateChanged(ItemEvent e) {

		cb.callback(id, this, new Boolean(cbox.getState()));
	}

	/**
	 * Setzt den Zustand.
	 *
	 * @param val Neuer aktueller Zustand.
	 */

	public void setValue(boolean val) {

		cbox.setState(val);
	}

	/**
	 * Liefert den Zustand.
	 *
	 * @return Gibt den aktuellen Zustand als boolean zurueck.
	 */

	public boolean getValue() {

		return(cbox.getState());
	}

	/**
	 * Aendert die Farben des Schalter-Objekts.
	 *
	 * @param fg   Vordergrund-Farbe der Box.
	 * @param bg   Hintergrund-Farbe der Box.
	 * @param sfg  Vordergrund-Farbe der internen Komponenten.
	 * @param sbg  Hintergrund-Farbe der internen Komponenten.
	 */

	public void chgColors(Color fg, Color bg, Color sfg, Color sbg) {

		chgBoxColors(fg, bg);

		cbox.setForeground(sfg == null ? FG : sfg);
		cbox.setBackground(sbg == null ? BG : sbg);
	}
}
