
package Controls;

import java.awt.*;

/** Ein Control-Objekt zur Anzeige eines Strings. */

public class Schild extends CtrlBox {

	private Label label;

	/**
	 * Erzeugt ein Schild-Objekt mit Name, Text, Rand und Farben.
	 *
	 * @param name	Name der Box.
	 * @param text	Angezeigter Text.
	 * @param rand	Rand oder kein Rand.
	 * @param fg	Vordergrund-Farbe der Box.
	 * @param bg	Hintergrund-Farbe der Box.
	 * @param sfg	Vordergrund-Farbe der internen Komponenten.
	 * @param sbg	Hintergrund-Farbe der internen Komponenten.
	 */

	public Schild(String name, String text, boolean rand, Color fg, Color bg, Color sfg, Color sbg) {

		super(name, rand ? RAND : NONE, fg, bg);

		label = new Label(text, Label.CENTER);

		label.setForeground(sfg == null ? FG : sfg);
		label.setBackground(sbg == null ? BG : sbg);

		addBag(label, 0, 0, 1, 1);
	}

	/** Erzeugt ein Schild-Objekt mit Default-Farben. */

	public Schild(String name, String text, boolean rand) {

		this(name, text, rand, null, null, null, null);
	}

	/** Erzeugt ein Schild-Objekt mit Rand. */

	public Schild(String name, String text, Color fg, Color bg, Color sfg, Color sbg) {

		this(name, text, true, fg, bg, sfg, sbg);
	}

	/** Erzeugt ein Schild-Objekt mit Rand und Default-Farben. */

	public Schild(String name, String text) {

		this(name, text, true, null, null, null, null);
	}

	/**
	 * Setzt den Text.
	 *
	 * @param text Neuer angezeigter Text.
	 */

	public void setValue(String text) {

		label.setText(text);
	}

	/**
	 * Liefert den Text.
	 *
	 * @return Gibt den angezeigten Text als String zurueck.
	 */

	public String getValue() {

		return(label.getText());
	}

	/**
	 * Aendert die Farben des Schild-Objekts.
	 *
	 * @param fg   Vordergrund-Farbe der Box.
	 * @param bg   Hintergrund-Farbe der Box.
	 * @param sfg  Vordergrund-Farbe der internen Komponenten.
	 * @param sbg  Hintergrund-Farbe der internen Komponenten.
	 */

	public void chgColors(Color fg, Color bg, Color sfg, Color sbg) {

		chgBoxColors(fg, bg);

		label.setForeground(sfg == null ? FG : sfg);
		label.setBackground(sbg == null ? BG : sbg);
	}
}
